
#ifndef __EXPORTS_H__
#define __EXPORTS_H__ 

/**
* EXMLPUBFUN, EXMLPUBVAR, EXMLCALL
*
* Macros which declare an exportable function, an exportable variable and
* the calling convention used for functions.
*
* Please use an extra block for every platform/compiler combination when
* modifying this, rather than overlong #ifdef lines. This helps
* readability as well as the fact that different compilers on the same
* platform might need different definitions.
*/

/**
* EXMLPUBFUN:
*
* Macros which declare an exportable function
*/
#define EXMLPUBFUN
/**
* EXMLPUBVAR:
*
* Macros which declare an exportable variable
*/
#define EXMLPUBVAR extern
/**
* EXMLCALL:
*
* Macros which declare the called convention for exported functions
*/
#define EXMLCALL
/**
* EXMLCDECL:
*
* Macro which declares the calling convention for exported functions that 
* use '...'.
*/
#define EXMLCDECL

/** DOC_DISABLE */

/* Windows platform with MS compiler */
/*#if defined(_WIN32) && defined(_MSC_VER)
#undef EXMLPUBFUN
#undef EXMLPUBVAR
#undef EXMLCALL
#undef EXMLCDECL
#if defined(IN_LIBXML) && !defined(LIBXML_STATIC)
#define EXMLPUBFUN __declspec(dllexport)
#define EXMLPUBVAR __declspec(dllexport)
#else
#define EXMLPUBFUN
#if !defined(LIBXML_STATIC)
#define EXMLPUBVAR __declspec(dllimport) extern
#else
#define EXMLPUBVAR extern
#endif
#endif
#if defined(LIBXML_FASTCALL)
#define EXMLCALL __fastcall
#else
#define EXMLCALL __cdecl
#endif
#define EXMLCDECL __cdecl
#if !defined _REENTRANT
#define _REENTRANT
#endif
#endif*/

/* Windows platform with Borland compiler */
/*#if defined(_WIN32) && defined(__BORLANDC__)
#undef EXMLPUBFUN
#undef EXMLPUBVAR
#undef EXMLCALL
#undef EXMLCDECL
#if defined(IN_LIBXML) && !defined(LIBXML_STATIC)
#define EXMLPUBFUN __declspec(dllexport)
#define EXMLPUBVAR __declspec(dllexport) extern
#else
#define EXMLPUBFUN
#if !defined(LIBXML_STATIC)
#define EXMLPUBVAR __declspec(dllimport) extern
#else
#define EXMLPUBVAR extern
#endif
#endif
#define EXMLCALL __cdecl
#define EXMLCDECL __cdecl
#if !defined _REENTRANT
#define _REENTRANT
#endif
#endif*/

/* Windows platform with GNU compiler (Mingw) */
/*#if defined(_WIN32) && defined(__MINGW32__)
#undef EXMLPUBFUN
#undef EXMLPUBVAR
#undef EXMLCALL
#undef EXMLCDECL
#if defined(IN_LIBXML) && !defined(LIBXML_STATIC)
#define EXMLPUBFUN __declspec(dllexport)
#define EXMLPUBVAR __declspec(dllexport)
#else
#define EXMLPUBFUN
#if !defined(LIBXML_STATIC)
#define EXMLPUBVAR __declspec(dllimport) extern
#else
#define EXMLPUBVAR extern
#endif
#endif
#define EXMLCALL __cdecl
#define EXMLCDECL __cdecl
#if !defined _REENTRANT
#define _REENTRANT
#endif
#endif*/

/* Cygwin platform, GNU compiler */
/*#if defined(_WIN32) && defined(__CYGWIN__)
#undef EXMLPUBFUN
#undef EXMLPUBVAR
#undef EXMLCALL
#undef EXMLCDECL
#if defined(IN_LIBXML) && !defined(LIBXML_STATIC)
#define EXMLPUBFUN __declspec(dllexport)
#define EXMLPUBVAR __declspec(dllexport)
#else
#define EXMLPUBFUN
#if !defined(LIBXML_STATIC)
#define EXMLPUBVAR __declspec(dllimport) extern
#else
#define EXMLPUBVAR
#endif
#endif
#define EXMLCALL __cdecl
#define EXMLCDECL __cdecl
#endif*/

/* Compatibility */
//#if !defined(LIBXML_DLL_IMPORT)
//#define LIBXML_DLL_IMPORT EXMLPUBVAR
//#endif 

#endif /* __EXPORTS_H__ */ 

