#pragma once

class WideToAnsi
{
public:
    WideToAnsi() {}
    WideToAnsi(const wchar_t *wide, int len = -1)
    {
        int wide_len = (len == -1) ? wcslen(wide) : len;
        convert(wide, wide_len);
    }

    int convert(const wchar_t *wide, int wide_len)
    {
        int symbols_count =  WideCharToMultiByte(CP_ACP, 0, wide, wide_len, NULL, 0, "?", NULL);
        if (m_convertBuffer.getSize() < (symbols_count+1))
            m_convertBuffer.alloc(symbols_count+1);
        char* buffer = (char*)m_convertBuffer.getData();
        WideCharToMultiByte(CP_ACP, 0, wide, wide_len, buffer, symbols_count, "?", NULL);
        buffer[symbols_count] = 0;
        return symbols_count;
    }

    operator const char* () 
    {
        return m_convertBuffer.getData();
    }

private:
    MemoryBuffer m_convertBuffer;
};

class AnsiToWide
{
public:
    AnsiToWide() {}
    AnsiToWide(const char *ansi, int len = -1)
    {
        int ansi_len = (len == -1) ? strlen(ansi) : len;
        convert(ansi, ansi_len);       
    }

    int convert(const char *ansi, int ansi_len)
    {
        int symbols_count =  MultiByteToWideChar(CP_ACP, 0, ansi, ansi_len, NULL, 0);
        int buffer_required = (symbols_count+1) * sizeof(wchar_t);
        if (m_convertBuffer.getSize() < buffer_required)
            m_convertBuffer.alloc(buffer_required);
     
        wchar_t* buffer = (wchar_t*)m_convertBuffer.getData();
        MultiByteToWideChar(CP_ACP, 0, ansi, ansi_len, buffer, buffer_required);
        buffer[symbols_count] = 0;
        return symbols_count;
    }

    operator const wchar_t* () 
    {
        return (wchar_t*)m_convertBuffer.getData();
    }

private:
    MemoryBuffer m_convertBuffer;
};

class AN2W
{
public:
    AN2W(const char* param)
    {
        AnsiToWide a2w(param);
        m_param.assign(a2w);
    }
    operator const tstring&()
    {
        return m_param; 
    }
    
private:
    tstring m_param;
};

class W2AN
{
public:
    W2AN(const tstring& param)
    {
        WideToAnsi w2a(param.c_str());
        m_param.assign(w2a);
    }
    operator const char*() 
    {
        return m_param.c_str(); 
    }
private:
    std::string m_param;
};
