#pragma once

class WideToUtf8
{
public:
    WideToUtf8() {}
    WideToUtf8(const wchar_t *wide, int len = -1)
    {
        int wide_len = (len == -1) ? wcslen(wide) : len;
        convert(wide, wide_len);
    }

    int convert(const wchar_t *wide, int wide_len)
    {
        int buffer_required =  WideCharToMultiByte(CP_UTF8, 0, wide, wide_len, NULL, 0, NULL, NULL);
        if (m_convertBuffer.getSize() < (buffer_required+1))
            m_convertBuffer.alloc(buffer_required+1);

        char* buffer = (char*)m_convertBuffer.getData();
        WideCharToMultiByte(CP_UTF8, 0, wide, wide_len, buffer, buffer_required, NULL, NULL);
        buffer[buffer_required] = 0;
        return buffer_required;
    }

    operator const char* () 
    {
        return m_convertBuffer.getData();
    }

private:
    MemoryBuffer m_convertBuffer;
};

class Utf8ToWide
{
public:
    Utf8ToWide() {}
    Utf8ToWide(const char *utf8, int len = -1)
    {
        int utf8_len = (len == -1) ? strlen(utf8) : len;
        convert(utf8, utf8_len);       
    }

    int convert(const char *utf8, int utf8_len)
    {
        int symbols_count =  MultiByteToWideChar(CP_UTF8, 0, utf8, utf8_len, NULL, 0);
        int buffer_required = (symbols_count+1) * sizeof(wchar_t);
        if (m_convertBuffer.getSize() < buffer_required)
            m_convertBuffer.alloc(buffer_required);
     
        wchar_t* buffer = (wchar_t*)m_convertBuffer.getData();
        MultiByteToWideChar(CP_UTF8, 0, utf8, utf8_len, buffer, buffer_required);
        buffer[symbols_count] = 0;
        return symbols_count;
    }

    operator const wchar_t* () 
    {
        return (wchar_t*)m_convertBuffer.getData();
    }

private:
    MemoryBuffer m_convertBuffer;
};

class U2W
{
public:
    U2W(const char* param)
    {
        Utf8ToWide u2w(param);
        m_param.assign(u2w);
    }
    operator const tstring&()
    {
        return m_param; 
    }
    
private:
    tstring m_param;
};

class W2U
{
public:
    W2U(const tstring& param)
    {
        WideToUtf8 w2u(param.c_str());
        m_param.assign(w2u);
    }
    operator const char*() 
    {
        return m_param.c_str(); 
    }
private:
    std::string m_param;
};
