#include "stdafx.h"
#include "incomingObjects.h"

Action::Action(const tstring& key, const tstring& value) : m_value(value)
{    
    m_compare.init(key);
}

Action::~Action()
{
}

bool Action::processing(MudViewStringBlock* block, tstring* newcmd)
{
    if (!m_compare.checkToCompare(block->string))
        return false;

    std::vector<tstring> params;            // values of params in key
    m_compare.getParameters(&params);
    int params_count = params.size();

    // parse value and generate result
    int pos = 0;
    ParamsHelper values(m_value);

    newcmd->clear();
    for (int i=0,e=values.getSize(); i<e; ++i)
    {
        newcmd->append( m_value.substr(pos, values.getFirst(i)-pos) );
        int id = values.getId(i);
        if (id < params_count)
            newcmd->append(params[id]);            
        pos = values.getLast(i);        
    }
    newcmd->append(m_value.substr(pos));    
    return true;
}

Sub::Sub(const tstring& key, const tstring& value) : m_value(value)
{
    m_compare.init(key);
}

Sub::~Sub()
{
}

bool Sub::processing(MudViewStringBlock* block )
{
    if (!m_compare.checkToCompare(block->string))
        return false;

    std::vector<tstring> params;            // values of params in key
    m_compare.getParameters(&params);
    int params_count = params.size();

    // parse value and generate result
    int pos = 0;
    ParamsHelper values(m_value);
    
    // calc replace string
    tstring replace;
    for (int i=0,e=values.getSize(); i<e; ++i)
    {
        replace.append( m_value.substr(pos, values.getFirst(i)-pos) );
        int id = values.getId(i);
        if (id < params_count)
           replace.append(params[id]);
        pos = values.getLast(i);        
    }
    replace.append(m_value.substr(pos));

    // replace
    m_compare.replace(replace, &block->string);

    // protect from cycles
    if (m_compare.checkToCompare(replace))
        return false;

    return true;
}


Highlight::Highlight(const tstring& key, const PropertiesHighlight& hl) : m_hl(hl)
{
    m_compare.init(key);
}

Highlight::~Highlight()
{
}

void Highlight::processing(MudViewString* string)
{
    for (int i=0,e=string->blocks.size(); i<e; ++i)
    {
        std::vector<MudViewStringBlock> &vb = string->blocks;
        MudViewStringBlock &b = vb[i];
        if (m_compare.checkToCompare(b.string))
        {
            CompareRange range;            
            m_compare.getRange(&range);

            // truncate block for 3 blocks
            
            // 1 block
            tstring str(b.string);
            MudViewStringParams p0(b.params);
            b.string = str.substr(0, range.begin);

            // 2 block
            MudViewStringBlock new_b;
            new_b.string = str.substr(range.begin, range.end-range.begin);

            MudViewStringParams &p = new_b.params;       // set highlights params for block
            p.use_ext_colors = 1;   
            p.ext_text_color = m_hl.textcolor;
            p.ext_bkg_color = m_hl.bkgcolor;            
            p.underline_status = m_hl.underlined;
            p.blink_status = m_hl.border;
            p.italic_status = m_hl.italic;
            vb.insert(vb.begin() + (i+1), new_b);
            i = i + 1;

            // 3 block (if it not empty)
            MudViewStringBlock end_b;
            end_b.string = str.substr(range.end);
            if (!end_b.string.empty())
            {
                end_b.params = p0;
                vb.insert(vb.begin() + (i+1), end_b);
            }

            e=string->blocks.size();
        }
    }   
}
