#include "stdafx.h"
#include "mudGameView.h"
#include "inputProcessor.h"
#include "mudViewParser.h"

void MudGameView::onNewCommandBarCmd(tstring cmd)
{
    WCHAR cmd_prefix = propData->cmd_prefix;
    InputProcessor ip(propData);
    ip.process(cmd);
    
    for (int i=0,e=ip.commands.size(); i<e; ++i)
    {
        tstring cmd = ip.commands[i]->full_command;
        if (!cmd.empty() && cmd.at(0) == cmd_prefix)
        {
            //it is system command for client (not game command)
            onProcessSytemCommand(cmd);      
        }
        else
        {
            // it is game command
            WCHAR br[2] = { 10, 0 };
            cmd.append(br);
            processIncoming(cmd.c_str(), cmd.length(), true);
            int len = m_convertorW2A.convert(cmd.c_str(), cmd.length());
            const char *text = m_convertorW2A;
            m_network.send((tbyte*)text, len);
        }
    }
}

bool MudGameView::onProcessHotkey(const tstring& key)
{   
    if (key.empty())
        return false;

    for (int i=0,e=propData->hotkeys.size(); i<e; ++i)
    {
        const PropertiesData::hotkey_data &hk = propData->hotkeys.get(i);
        if (hk.key == key)
        {
            onNewCommandBarCmd(hk.value);
            return true;
        }
    }
    return false;
}

void MudGameView::onProcessSytemCommand(const tstring& cmd)
{
    m_syscmds.process(cmd);
}

void MudGameView::onNewNetworkData()
{
   DataQueue* data = m_network.receive();
   int text_len =  data->getSize();
   if (text_len == 0)
       return;

   char *server_text = (char*)data->getData();
   int len = m_convertorA2W.convert(server_text, text_len);
   data->truncate(text_len);
   processIncoming(m_convertorA2W, len);
}

void MudGameView::onDisconnectNetwork()
{
    siLostConnection();
}

void MudGameView::processIncoming(const WCHAR* text, int text_len, bool skip_actions)
{
   // parse incoming text
   parseData parse_data;
   m_parser.parse(text, text_len, &parse_data);

   // accamulate last string in one
   m_view.accLastString(&parse_data);

   // collect strings in parse_data in one with same colors params
   ParseDataCollector pc;
   pc.process(&parse_data);

   // array for new cmds from actions
   std::vector<tstring> new_cmds;

   // process actions
   if (!skip_actions)
   {
       // process actions    
       m_incomingProcessor.processActions(&parse_data, &new_cmds);    

       // process substs
       m_incomingProcessor.processSubs(&parse_data);

       // process highlights
       m_incomingProcessor.processHighlights(&parse_data);
   }

   // send processed text to view
   m_view.addText(&parse_data);

   // free unused strings
   m_parser.free(&parse_data, true);

   // process actions' result
   for (int i=0,e=new_cmds.size(); i<e; ++i)
         onNewCommandBarCmd(new_cmds[i]); 
}
