#include "stdafx.h"
#include "propertiesPages/propertiesData.h"
#include "mudView.h"
#pragma warning(disable: 4996)

MudView::MudView(PropertiesData *data) : 
propData(data),
m_palette(data),
m_line_heigth(0),
m_lines_count(0),
m_last_visible_line(-1),
m_last_string_updated(false)
{
}

MudView::~MudView()
{
}

void MudView::accLastString(parseData *parse_data)
{
    if (m_strings.empty() ||
        parse_data->strings.empty() ||
        !parse_data->update_prev_string) return;

    MudViewString *string = parse_data->strings[0];
    int prev = m_strings.size() - 1;
    m_strings[prev]->moveBlocks(string);    
    parse_data->free_strings.push_back(string);    
    parse_data->strings[0] = m_strings[prev];
    m_strings.pop_back();                                   // remove prev string from view
}

void MudView::addText(parseData* parse_data)
{   
    if (parse_data->strings.empty())
        return;

    for (int i=0,e=parse_data->strings.size(); i<e; ++i)
    {
        MudViewString *string = parse_data->strings[i];
        std::vector<MudViewStringBlock> &b = string->blocks;
        for (int j=0,je=b.size(); j<je; ++j)
        {
            const tstring &s = b[j].string;
            GetTextExtentPoint32(m_dc, s.c_str(), s.length(), &b[j].size);
        }
        m_strings.push_back(string);
    }
    parse_data->strings.clear();
    checkLimit(parse_data);

    int new_visible_line = m_strings.size() - 1;
    updateScrollbar(new_visible_line);
    Invalidate();
}

void MudView::updateProps()
{
    m_dc = GetDC();
    if (!m_background.IsNull())
        m_background.DeleteObject();
    m_background = CreateSolidBrush(propData->bkgnd);

    if (!m_font.IsNull())
        m_font.DeleteObject();
    LOGFONT logfont;
    propData->initLogFont(m_hWnd, &logfont);
    m_font.CreateFontIndirect(&logfont);

    m_dc.SelectFont(m_font);
    SIZE sz = {0,0};
    GetTextExtentPoint32(GetDC(), L"W", 1, &sz);
    m_line_heigth = sz.cy;

    if (!m_underlined_font.IsNull())
        m_underlined_font.DeleteObject();
    logfont.lfUnderline = 1;
    m_underlined_font.CreateFontIndirect(&logfont);

    if (!m_italic_font.IsNull())
        m_italic_font.DeleteObject();
    logfont.lfUnderline = 0;
    if (logfont.lfItalic)
        logfont.lfItalic = 0;
    else
        logfont.lfItalic = 1;
    m_italic_font.CreateFontIndirect(&logfont);

    if (!m_italic_underlined_font.IsNull())
         m_italic_underlined_font.DeleteObject();
    logfont.lfUnderline = 0;
    m_italic_underlined_font.CreateFontIndirect(&logfont);

    for (int i=0,e=m_strings.size(); i<e; ++i)
    {
        MudViewString *string = m_strings[i];
        std::vector<MudViewStringBlock> &b = string->blocks;
        for (int j=0,je=b.size(); j<je; ++j)
        {
            const tstring &s = b[j].string;
            MudViewStringParams &p = b[j].params;
            if (p.underline_status || p.italic_status)
            {
                 if (p.underline_status && p.italic_status)
                     m_dc.SelectFont(m_italic_underlined_font);
                 else if (p.underline_status)                         
                     m_dc.SelectFont(m_underlined_font);
                 else
                     m_dc.SelectFont(m_italic_font);
            }
            GetTextExtentPoint32(m_dc, s.c_str(), s.length(), &b[j].size);
            if (p.underline_status || p.italic_status)
                m_dc.SelectFont(m_font);
        } 
    }
    Invalidate();
}

void MudView::checkLimit(parseData* parse_data)
{
    int size = m_strings.size();
    if (size > propData->view_history_size)
    {
        size = size - propData->view_history_size;
        parseDataStrings &f = parse_data->free_strings;
        f.insert(f.end(), m_strings.begin(), m_strings.begin()+size);
        m_strings.erase(m_strings.begin(), m_strings.begin()+size);
    }    
}

void MudView::renderView()
{
    RECT pos; GetClientRect(&pos);
    CPaintDC dc(m_hWnd);
    CMemoryDC mdc(dc, pos);
    mdc.FillRect(&pos, m_background);
    mdc.SelectFont(m_font);
    if (m_strings.empty())
        return;
    if (m_last_visible_line == -1)
        return;

    int index = m_last_visible_line;
    int count = m_lines_count + 1;
    int y = pos.bottom;                
    while (index >= 0 && count > 0)
    {
        renderString(&mdc, m_strings[index], 0, y);
        index--; count--;
        y = y - m_line_heigth;
    }
}

void MudView::renderString(CDC *dc, MudViewString *s, int left_x, int bottom_y)
{
    std::vector<MudViewStringBlock> &b = s->blocks;
    RECT pos = { left_x, bottom_y-m_line_heigth, left_x, bottom_y };   
    for (int i=0,e=b.size(); i<e; ++i)
    {
        pos.left = pos.right;
        pos.right += b[i].size.cx;

        MudViewStringParams &p = b[i].params;
        COLORREF text_color = p.ext_text_color;
        COLORREF bkg_color = p.ext_bkg_color;
        if (!p.use_ext_colors)
        {
            tbyte txt = p.text_color;
            tbyte bkg = p.bkg_color;
            if (p.reverse_video) { tbyte x = txt; txt = bkg; bkg = txt; }    
            if (txt >= 0 && txt <= 7 && p.intensive_status) {
                    txt += 8;
            }
            text_color = m_palette.getColor(txt);
            if (bkg == 0)
                bkg_color = propData->bkgnd;
            else
                bkg_color = m_palette.getColor(bkg);
        }

        dc->SetTextColor(text_color);
        dc->SetBkColor(bkg_color);

        if (p.underline_status || p.italic_status)
        {
            if (p.underline_status && p.italic_status)
                dc->SelectFont(m_italic_underlined_font);
            else if (p.underline_status)                         
                dc->SelectFont(m_underlined_font);
            else
                dc->SelectFont(m_italic_font);
        }
        
        const tstring &s = b[i].string;
        dc->DrawText(s.c_str(), s.length(), &pos, DT_CENTER|DT_SINGLELINE|DT_VCENTER);
        
        if (p.underline_status || p.italic_status)
            dc->SelectFont(m_font);

        if (p.blink_status)
        {
                CPen pen;
                pen.CreatePen(PS_SOLID, 1, text_color);
                HPEN oldpen = dc->SelectPen(pen);
                dc->MoveTo(pos.left, pos.top, NULL); 
                dc->LineTo(pos.right-1, pos.top);
                dc->LineTo(pos.right-1, pos.bottom-1);
                dc->LineTo(pos.left, pos.bottom-1);
                dc->LineTo(pos.left, pos.top);
                dc->SelectPen(oldpen);
        }
    }
}

void MudView::initRenerParams()
{
    RECT rc; GetClientRect(&rc);
    if (rc.right <= 0 || rc.bottom <= 0)        // check width and heigth
    {
        m_lines_count = 0;
        return;
    }
    m_lines_count = rc.bottom / m_line_heigth;
}

void MudView::updateScrollbar(int new_visible_line)
{
    int lines = m_strings.size();
    if (lines <= m_lines_count || m_lines_count == 0)
    {
        SetScrollRange(SB_VERT, 0, 0);
        SetScrollPos(SB_VERT, 0);
        m_last_visible_line = lines-1;
    }
    else
    {
        int max_sb = lines - m_lines_count;
        SetScrollRange(SB_VERT, 0, max_sb, FALSE);

        int max_visible = lines - 1;
        int min_visible = m_lines_count - 1;
        
        if (new_visible_line < min_visible)
            new_visible_line = min_visible;
        if (new_visible_line > max_visible)
            new_visible_line = max_visible;

        m_last_visible_line = new_visible_line;
        int pos_sb = new_visible_line - m_lines_count + 1;        
        SetScrollPos(SB_VERT, pos_sb);
    }
}

void MudView::setScrollbar(DWORD position)
{
    int thumbpos = HIWORD(position);
    int action = LOWORD(position);
    
    int visible_line = m_last_visible_line;
    switch(action) {
    case SB_LINEUP:
         visible_line = visible_line - 1;
    break;
    case SB_LINEDOWN:
         visible_line = visible_line + 1;
    break;
    case SB_PAGEUP:
        visible_line = visible_line - m_lines_count;
    break;
    case SB_PAGEDOWN:
        visible_line = visible_line + m_lines_count;
    break;
    case SB_THUMBTRACK:
    case SB_THUMBPOSITION:        
        visible_line = (thumbpos + m_lines_count) - 1;
    break;
    }
    updateScrollbar(visible_line);
    Invalidate();    
}

void MudView::mouseWheel(WORD position)
{
    int direction = (position & 0x8000) ? 3 : -3;
    int visible_line = m_last_visible_line + direction;
    updateScrollbar(visible_line);
    Invalidate();
}
