#pragma once

#include "propertyList.h"
#include "propertiesData.h"

#include "propertiesData.h"
#include "propertiesPageAliases.h"
#include "propertiesPageActions.h"
#include "propertiesPageHighlights.h"
#include "propertiesPageHotkeys.h"
#include "propertiesPageSubs.h"
#include "propertiesPageColors.h"
#include "propertiesPageCommon.h"

class PropertiesDlg :  public CDialogImpl<PropertiesDlg>
{
    PropertyAliases m_aliases;
    PropertyActions m_actions;
    PropertyHighlights m_highlights;
    PropertyHotkeys m_hotkeys;
    PropertySubs m_subs;
    PropertyColors m_colors;
    PropertyCommon m_common;
  
    int m_width;
    int m_heigth;
    int m_bar_width;
    int m_additional_border;
    CBevelLine m_line;
    HWND m_currentPage;

public:
   enum { IDD = IDD_PROPERTY_BASE };
   PropertiesDlg(PropertiesData* props) : m_aliases(props), m_actions(props), m_highlights(props), m_hotkeys(props), m_subs(props), m_colors(props), m_common(props),
       m_width(0), m_heigth(0), m_bar_width(0), m_additional_border(4), m_currentPage(NULL)
   {
       assert(props);
   }

private:
    BEGIN_MSG_MAP(PropertiesDlg)
        MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
        COMMAND_ID_HANDLER(IDOK, OnCloseCmd)
        COMMAND_ID_HANDLER(IDCANCEL, OnCloseCmd)
        COMMAND_ID_HANDLER(IDC_BUTTON_PROP_ALIASES, onAliases)
        COMMAND_ID_HANDLER(IDC_BUTTON_PROP_ACTIONS, onActions)
        COMMAND_ID_HANDLER(IDC_BUTTON_PROP_HIGHLIGHTS, onHightlights)
        COMMAND_ID_HANDLER(IDC_BUTTON_PROP_HOTKEYS, onHotkeys)
        COMMAND_ID_HANDLER(IDC_BUTTON_PROPS_SUBS, onSubs)
        COMMAND_ID_HANDLER(IDC_BUTTON_PROP_COLORS, onColors)
        COMMAND_ID_HANDLER(IDC_BUTTON_PROP_COMMON, onCommon)
    END_MSG_MAP()

    LRESULT onAliases(WORD, WORD id, HWND, BOOL&) { selectPage(m_aliases); return 0; }
    LRESULT onActions(WORD, WORD id, HWND, BOOL&) { selectPage(m_actions); return 0; }
    LRESULT onHightlights(WORD, WORD id, HWND, BOOL&) { selectPage(m_highlights); return 0; }
    LRESULT onHotkeys(WORD, WORD id, HWND, BOOL&) { selectPage(m_hotkeys); return 0; }
    LRESULT onSubs(WORD, WORD id, HWND, BOOL&) { selectPage(m_subs); return 0; }
    LRESULT onColors(WORD, WORD id, HWND, BOOL&) { selectPage(m_colors); return 0; }
    LRESULT onCommon(WORD, WORD id, HWND, BOOL&) { selectPage(m_common); return 0; }

    void selectPage(HWND page)
    {
        if (page == m_currentPage)
            return;
        if (m_currentPage != NULL)
            ::ShowWindow(m_currentPage, SW_HIDE);
        m_currentPage = page;
        ::ShowWindow(page, SW_SHOW);
        ::SetFocus(page);
    }

    void setPosition(HWND page)
    {
        RECT pos;
        ::GetClientRect(page, &pos);
        int width = pos.right;
        int heigth = pos.bottom;

        pos.left = (m_width - width) / 2;
        pos.top = 0;
        pos.right = pos.left + width;
        pos.bottom = pos.top + heigth;
        ::MoveWindow(page, pos.left, pos.top, width, heigth, FALSE);
    }
       
	LRESULT OnInitDialog(UINT, WPARAM, LPARAM, BOOL&)
	{        
        // add pages
        m_aliases.Create(m_hWnd);
        m_actions.Create(m_hWnd);
        m_highlights.Create(m_hWnd);
        m_hotkeys.Create(m_hWnd);
        m_subs.Create(m_hWnd);
        m_colors.Create(m_hWnd);
        m_common.Create(m_hWnd);

        // line delimeter
        m_line.SubclassWindow(GetDlgItem(IDC_STATIC_BLLINE));

        // calc size of freespace for pages
        RECT pos;
        GetClientRect(&pos);
        m_heigth = pos.bottom;
        m_width = pos.right;

        // set position of vertical delimeter
        GetWindowRect(&pos);
        int left_pos = pos.left;
        m_line.GetWindowRect(&pos);
        left_pos = pos.left - left_pos;
        m_width = left_pos;
      
        // calc main window size + move it
        GetWindowRect(&pos);
        int width = pos.right - pos.left;
        int heigth = pos.bottom - pos.top;

        DWORD screen_width = GetSystemMetrics(SM_CXSCREEN);
        DWORD screen_heigth = GetSystemMetrics(SM_CYSCREEN);
        pos.left = (screen_width - width) / 2;
        pos.top = (screen_heigth - heigth) / 2;
        pos.right = pos.left + width;
        pos.bottom = pos.top + heigth;
        MoveWindow(&pos);
        CenterWindow(GetParent());
               
        setPosition(m_aliases);
        setPosition(m_actions);
        setPosition(m_highlights);
        setPosition(m_hotkeys);
        setPosition(m_subs);
        setPosition(m_colors);
        setPosition(m_common);

        selectPage(m_aliases);
		return TRUE;
	}

	LRESULT OnCloseCmd(WORD, WORD wID, HWND, BOOL&)
	{
		EndDialog(wID);
		return 0;
	}
};
