#include "stdafx.h"
#include "propertiesManager.h"
#include "profileHelpers.h"

PropertiesManager::PropertiesManager()
{
}

PropertiesManager::~PropertiesManager()
{
}

bool PropertiesManager::init()
{
    ProfilesGroupList groups;
    if (!groups.init())
        return false;
    int last = groups.getLast();
    groups.getName(last, &m_configName);   
    return true;
}

bool PropertiesManager::loadProfile()
{    
    if (loadSettings() && loadProfileData())
    {
        loadHistory();
        return true;
    }
    return false;
}

bool PropertiesManager::loadSettings()
{
    xml::node sd;
    if (!loadFromFile(sd, L"settings.xml"))
        return false;

    std::string profile;
    xml::request r(sd, "profile");
    if (r.size() != 0)          
        r[0].gettext(&profile);
    if (profile.empty())
        return false;

    m_profileName.assign( Utf8ToWide(profile.c_str()) );
    sd.deletenode();
    return true;
}

bool PropertiesManager::loadHistory()
{
    m_propData.cmd_history.clear();
    xml::node hd;
    if (!loadFromFile(hd, L"history.xml"))
        return false;

    Utf8ToWide u2w;
    xml::request r(hd, "cmd");
    for (int i=0,e=r.size(); i<e; ++i)
    {
        std::string cmd;
        r[i].gettext(&cmd);
        if (!cmd.empty())
        {
            u2w.convert(cmd.c_str(), cmd.length());
            m_propData.cmd_history.push_back(tstring(u2w));
        }
        if (i>MAX_CMD_HISTORY_SIZE)
            break;
    }
    hd.deletenode();
    return true;
}

bool PropertiesManager::loadProfileData()
{
    m_propData.initAllDefault();
    tstring profile(L"profiles\\");
    profile.append(m_profileName);
    profile.append(L".xml");

    xml::node sd;
    if (!loadFromFile(sd, profile))
        return false;
    
    xml::request vs(sd, "viewsize");
    if (vs.size() != 0)
    {
        int size = 0;
        if (vs[0].getattribute("value", &size))
        {
            if (size < MIN_VIEW_HISTORY_SIZE)
                size = MIN_VIEW_HISTORY_SIZE;
            if (size > MAX_VIEW_HISTORY_SIZE)
                size = MAX_VIEW_HISTORY_SIZE;
            m_propData.view_history_size = size;
        }
    }

    xml::request cs(sd, "cmdsize");
    if (cs.size() != 0)
    {
        int size = 0;
        if (cs[0].getattribute("value", &size))
        {
            if (size < MIN_CMD_HISTORY_SIZE)
                size = MIN_CMD_HISTORY_SIZE;
            if (size > MAX_CMD_HISTORY_SIZE)
                size = MAX_CMD_HISTORY_SIZE;
            m_propData.cmd_history_size = size;
        }
    }

    xml::request colors(sd, "colors/color");
    for (int i=0,e=colors.size(); i<e; ++i)
    {
        std::string name; COLORREF clr;
        if (loadRgbColor(colors[i], &name, &clr))
        {
            if (name == "background")
                m_propData.bkgnd = clr;
            else  
            {
                int id = 0;
                if (a2int(name, &id) && id >= 0 && id <= 15)
                    m_propData.colors[id] = clr;
            }
        }
    }

    xml::request font(sd, "font");
    if (font.size())
    {
        xml::node f = font[0];
        f.getattribute("italic", &m_propData.font_italic);
        f.getattribute("bold", &m_propData.font_bold);
        f.getattribute("heigth", &m_propData.font_heigth);
        std::string font_name;
        f.getattribute("name", &font_name);
        if (!font_name.empty())
            m_propData.font_name = Utf8ToWide(font_name.c_str());
    }
    
    loadArray(sd, "actions/action", &m_propData.actions);
    loadArray(sd, "aliases/alias", &m_propData.aliases);
    loadArray(sd, "hotkeys/hotkey", &m_propData.hotkeys);
    loadArray(sd, "subs/sub", &m_propData.subs);
    
    xml::request hd(sd, "highlights/highlight");
    Utf8ToWide u2w;
    for (int i=0,e=hd.size(); i<e; ++i)
    {
        std::string key;
        PropertiesHighlight hl;
        if (hd[i].getattribute("key", &key) &&
            hd[i].getattribute("underline", &hl.underlined) &&
            hd[i].getattribute("border", &hl.border) &&
            hd[i].getattribute("italic", &hl.italic))
        {
            u2w.convert(key.c_str(), key.length());
            tstring _key(u2w);
            if (m_propData.highlights.exist(_key)) 
                continue;
            if (hl.underlined != 0) hl.underlined = 1;
            if (hl.border != 0) hl.border = 1;
            if (hl.italic != 0) hl.italic = 1;

            COLORREF color = 0;
            std::string name;
            xml::request colors(hd[i], "color");
            for (int j=0,je=colors.size(); j<je; ++j)
            {
                if (loadRgbColor(colors[j], &name, &color))
                {
                    if (name == "text")
                        hl.textcolor = color;
                    if (name == "background")
                        hl.bkgcolor = color;
                }
            }
            m_propData.highlights.add(-1, _key, hl);
        }
    }
     
    sd.deletenode();
    return true;
}

bool PropertiesManager::saveProfile()
{
    saveHistory();
    if (!saveProfileData())
        return false;
    if (!saveSettings())
        return false;
    return true;
}

bool PropertiesManager::saveProfileData()
{
    xml::node sd;
    sd.createnew(NULL, "profile");

    xml::node n = sd.createsubnode("viewsize");
    n.setattribute("value", m_propData.view_history_size);    
    xml::node n2 = sd.createsubnode("cmdsize");
    n2.setattribute("value", m_propData.cmd_history_size);
    
    xml::node c = sd.createsubnode("colors");
    saveRgbColor(c, "background", m_propData.bkgnd);
    for (int i=0; i<16; ++i) {
         char buffer[4];
         itoa(i, buffer, 10);
         saveRgbColor(c, buffer, m_propData.colors[i]);
    }

    xml::node f = sd.createsubnode("font");
    f.setattribute("italic", m_propData.font_italic);
    f.setattribute("bold", m_propData.font_bold);
    f.setattribute("heigth", m_propData.font_heigth);
    WideToUtf8 font_name(m_propData.font_name.c_str());
    f.setattribute("name", std::string(font_name));

    xml::node actions = sd.createsubnode("actions");
    saveArray(actions, "action", m_propData.actions);

    xml::node aliases = sd.createsubnode("aliases");
    saveArray(aliases, "alias", m_propData.aliases);

    xml::node hotkeys = sd.createsubnode("hotkeys");
    saveArray(hotkeys, "hotkey", m_propData.hotkeys);

    xml::node subs = sd.createsubnode("subs");
    saveArray(subs, "sub", m_propData.subs);

    xml::node hd = sd.createsubnode("highlights");
    {
        WideToUtf8 w2u;
        for (int i=0,e=m_propData.highlights.size(); i<e; ++i)
        {
            const PropertiesData::highlight_data& hl = m_propData.highlights.get(i);
            xml::node data_node = hd.createsubnode("highlight");
            w2u.convert(hl.key.c_str(), hl.key.length());
            data_node.setattribute("key", std::string(w2u));
            data_node.setattribute("underline", hl.value.underlined);
            data_node.setattribute("border", hl.value.border);
            data_node.setattribute("italic", hl.value.italic);
            saveRgbColor(data_node, "text", hl.value.textcolor);
            saveRgbColor(data_node, "background", hl.value.bkgcolor);
        }
    }

    tstring config(L"profiles\\");
    config.append(m_profileName);
    config.append(L".xml");

    bool result = saveToFile(sd, config);
    sd.deletenode();
    return result;
}

bool PropertiesManager::saveHistory()
{
    std::vector<tstring> &h = m_propData.cmd_history;
    if (h.empty())
        return true;

    xml::node hd;
    hd.createnew(NULL, "history");
    WideToUtf8 w2u;

    for (int i=0, e=h.size(); i<e; ++i) 
    {
        xml::node n = hd.createsubnode("cmd");
        w2u.convert(h[i].c_str(), h[i].length());
        n.settext(std::string(w2u));
    }
    bool result = saveToFile(hd, L"history.xml");
    hd.deletenode();
    return result;
}

bool PropertiesManager::saveSettings()
{
    xml::node sd;
    sd.createnew(NULL, "settings");
    xml::node n = sd.createsubnode("profile");
    std::string pname( WideToUtf8(m_profileName.c_str()) );
    n.settext(pname);
    bool result = saveToFile(sd, L"settings.xml");
    sd.deletenode();
    return result;
}
//----------------------------------------------------------------------------
bool PropertiesManager::loadRgbColor(xml::node n, std::string* name, COLORREF* color)
{
    std::string cn;
    if (!n.getattribute("id", &cn))
        return false;
    int r=0; int g=0; int b=0;
    if (!n.getattribute("r", &r) ||
        !n.getattribute("g", &g) ||
        !n.getattribute("b", &b)
       ) return false;        
    name->assign(cn);
    *color = RGB(r,g,b);
    return true;
}

void PropertiesManager::loadArray(xml::node n, const std::string& name, props_array* value)
{
    xml::request r(n, name.c_str());
    Utf8ToWide u2w;
    for (int i=0,e=r.size(); i<e; ++i)
    {
        std::string key, val;
        if (r[i].getattribute("key", &key) &&
            r[i].getattribute("value", &val) &&
            !key.empty())
        {
            u2w.convert(key.c_str(), key.length());
            tstring _key(u2w);
            if (value->exist(_key)) continue;
            u2w.convert(val.c_str(), val.length());
            value->add(-1, _key, tstring(u2w));            
        }
    }   
}
//----------------------------------------------------------------------------
void PropertiesManager::saveRgbColor(xml::node parent, const std::string& name, COLORREF color)
{
    xml::node n = parent.createsubnode("color");
    n.setattribute("id", name.c_str());
    n.setattribute("r", GetRValue(color));
    n.setattribute("g", GetGValue(color));
    n.setattribute("b", GetBValue(color));    
}

void PropertiesManager::saveArray(xml::node parent, const std::string& name, const props_array& value)
{
    WideToUtf8 w2u;
    for (int i=0,e=value.size(); i<e; ++i)
    {
        const props_array::el &data = value.get(i);
        xml::node data_node = parent.createsubnode(name.c_str());
        w2u.convert(data.key.c_str(), data.key.length());
        data_node.setattribute("key", std::string(w2u));
        w2u.convert(data.value.c_str(), data.value.length());
        data_node.setattribute("value", std::string(w2u));        
    }
}

bool PropertiesManager::loadFromFile(xml::node& node, const tstring& file)
{
    tstring config(m_configName);
    config.append(L"\\");
    config.append(file);
    std::string config_ansi( WideToAnsi(config.c_str()) );
    return (node.load(NULL, config_ansi.c_str())) ? true : false;        
}

bool PropertiesManager::saveToFile(xml::node node, const tstring& file)
{
    tstring config(m_configName);
    config.append(L"\\");
    config.append(file);
    std::string config_ansi( WideToAnsi(config.c_str()) );
    return (node.save(config_ansi.c_str())) ? true : false;
}

//----------------------------------------------------------------------------
bool PropertiesManager::createNewProfile(const tstring& name)
{
    m_propData.initAllDefault();
    m_profileName = name;
    bool result = saveProfileData();
    saveSettings();
    return result;
}

bool PropertiesManager::createCopyProfile(const tstring& from, const tstring& name)
{
    m_profileName = from;
    if (!loadProfileData())
        { m_profileName = name; return false; }

    m_profileName = name;
    bool result = saveProfileData();
    saveSettings();
    return result;
}

bool PropertiesManager::loadNewProfile(const tstring& group, const tstring& name)
{   
    m_configName = group;
    m_profileName = name;
    if (loadProfileData())
    {
        loadHistory();
        saveSettings();
        return true;
    }
    return false;
}

bool PropertiesManager::createNewProfile(const tstring& group, const tstring& name)
{
    m_propData.initAllDefault();
    m_configName = group;
    m_profileName = name;

    ProfileDirHelper dh;
    if (!dh.makeDir(L"", m_configName) ||
        !dh.makeDir(m_configName, L"profiles") )
    return false;
    bool result = saveProfileData();
    saveSettings();
    return result;
}