#pragma once

class PropertyActions :  public CDialogImpl<PropertyActions>
{    
    PropertiesData *propData;
    PropertyListCtrl m_list;
    CBevelLine m_bl1;
    CBevelLine m_bl2;
    CEdit m_pattern;
    CEdit m_action;
    CButton m_add;
    CButton m_del;
    CButton m_up;
    CButton m_down;
    bool m_block;

public:
    enum { IDD = IDD_PROPERTY_ACTIONS };
    PropertyActions(PropertiesData *data) : propData(data), m_block(false) {}

private:
    BEGIN_MSG_MAP(PropertyActions)
       MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
       COMMAND_ID_HANDLER(IDC_BUTTON_ACTION_ADD, OnAddElement)
       COMMAND_ID_HANDLER(IDC_BUTTON_ACTION_DEL, OnDeleteElement)
       COMMAND_ID_HANDLER(IDC_BUTTON_ACTION_UP, OnUpElement)
       COMMAND_ID_HANDLER(IDC_BUTTON_ACTION_DOWN, OnDownElement)
       COMMAND_HANDLER(IDC_EDIT_ACTION_PATTERN, EN_CHANGE, OnPatternEditChanged)
       NOTIFY_HANDLER(IDC_LIST_ACTIONS, LVN_ITEMCHANGED, OnListItemChanged)
       NOTIFY_HANDLER(IDC_LIST_ACTIONS, NM_SETFOCUS, OnListItemChanged)
       NOTIFY_HANDLER(IDC_LIST_ACTIONS, NM_KILLFOCUS, OnListKillFocus)

    END_MSG_MAP()

    LRESULT OnAddElement(WORD, WORD, HWND, BOOL&)
    {
        tstring pattern, action;
        getWindowText(m_pattern, &pattern);
        getWindowText(m_action, &action);
        int index = propData->actions.find(pattern);
        if (index == -1)
        {
            int pos = m_list.GetItemCount();
            m_list.AddItem(pos, 0, pattern.c_str());
            m_list.AddItem(pos, 1, action.c_str());
        }
        else
        {
            m_block = true;
            m_list.SetItem(index, 0, LVIF_TEXT, pattern.c_str(), 0, 0, 0, NULL);
            m_list.SetItem(index, 1, LVIF_TEXT, action.c_str(), 0, 0, 0, NULL);            
        }
        propData->actions.add(index, pattern, action);
        if (index == -1)
            index = m_list.GetItemCount()-1;
        m_list.SelectItem(index);
        return 0;
    }

    LRESULT OnDeleteElement(WORD, WORD, HWND, BOOL&)
    {   
        int index = m_list.GetSelectedIndex();
        m_list.DeleteItem(index);        
        propData->actions.del(index);
        return 0;
    }

    void swapItems(int index1, int index2)
    {
        const PropertiesData::alias_data& i1 = propData->actions.get(index1);
        const PropertiesData::alias_data& i2 = propData->actions.get(index2);
        m_list.SetItemText(index1, 0, i2.key.c_str());
        m_list.SetItemText(index1, 1, i2.value.c_str());
        m_list.SetItemText(index2, 0, i1.key.c_str());
        m_list.SetItemText(index2, 1, i1.value.c_str());
        propData->actions.swap(index1, index2);
    }
    
    LRESULT OnUpElement(WORD, WORD, HWND, BOOL&)
    {
        int index = m_list.GetSelectedIndex();
        if (index > 0)
        {
            swapItems(index, index-1);
            m_list.SelectItem(index-1);
        }
        return 0;
    }

    LRESULT OnDownElement(WORD, WORD, HWND, BOOL&)
    {
        int index = m_list.GetSelectedIndex();
        int last = m_list.GetItemCount() - 1;
        if (index >= 0 && index != last)
        {
            swapItems(index, index+1);
            m_list.SelectItem(index+1);
        }
        return 0;
    }
    
    LRESULT OnPatternEditChanged(WORD, WORD, HWND, BOOL&)
    {
        int string_len = m_pattern.GetWindowTextLength();
        m_add.EnableWindow(string_len == 0 ? FALSE : TRUE);
        return 0;
    }

    LRESULT OnListItemChanged(int , LPNMHDR , BOOL&)
    {
        int item_selected = m_list.GetSelectedIndex();
        if (item_selected == -1)
        {
            m_del.EnableWindow(FALSE);
            m_up.EnableWindow(FALSE);
            m_down.EnableWindow(FALSE);
        }
        else
        {
            m_del.EnableWindow(TRUE);
            m_up.EnableWindow(TRUE);
            m_down.EnableWindow(TRUE);
        }

        if (item_selected != -1 && !m_block)
        {
            const PropertiesData::action_data& action = propData->actions.get(item_selected);
            m_pattern.SetWindowText(action.key.c_str());
            m_action.SetWindowText(action.value.c_str());            
        }      
        m_block = false;
        return 0;
    }    

    LRESULT OnListKillFocus(int , LPNMHDR , BOOL&)
    {
        if (GetFocus() != m_del)        
            m_del.EnableWindow(FALSE);          
        return 0;
    }

    LRESULT OnInitDialog(UINT, WPARAM, LPARAM, BOOL&)
	{
        m_pattern.Attach(GetDlgItem(IDC_EDIT_ACTION_PATTERN));
        m_action.Attach(GetDlgItem(IDC_EDIT_ACTION));
        m_add.Attach(GetDlgItem(IDC_BUTTON_ACTION_ADD));
        m_del.Attach(GetDlgItem(IDC_BUTTON_ACTION_DEL));
        m_up.Attach(GetDlgItem(IDC_BUTTON_ACTION_UP));
        m_down.Attach(GetDlgItem(IDC_BUTTON_ACTION_DOWN));
        m_list.Attach(GetDlgItem(IDC_LIST_ACTIONS));
        m_list.addColumn(L"", 50);
        m_list.addColumn(L"Action", 50);
        m_list.SetExtendedListViewStyle( m_list.GetExtendedListViewStyle() | LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT);
        m_bl1.SubclassWindow(GetDlgItem(IDC_STATIC_ACBL1));
        m_bl2.SubclassWindow(GetDlgItem(IDC_STATIC_ACBL2));
        m_add.EnableWindow(FALSE);
        m_del.EnableWindow(FALSE);
        m_up.EnableWindow(FALSE);
        m_down.EnableWindow(FALSE);

        for (int i=0,e=propData->actions.size(); i<e; ++i)
        {
            const PropertiesData::action_data& action = propData->actions.get(i);
            m_list.AddItem(i, 0, action.key.c_str());
            m_list.AddItem(i, 1, action.value.c_str());
        }
        return 0;
    }
};

