#include "stdafx.h"
#include "pcreHelper.h"

#ifdef _UNICODE
#ifdef _DEBUG
#pragma comment(lib, "pcre16d.lib")
#else
#pragma comment(lib, "pcre16.lib")
#endif
#endif

Pcre16::Pcre16(): m_ph(NULL), m_pe(NULL)
{
}

Pcre16::~Pcre16()
{
    clear();
}

bool Pcre16::setRegExp(const tstring& regexp, bool extra)
{
    clear();
    const unsigned short *reg = (unsigned short*)regexp.c_str();
    const char *error = NULL;
    int error_offset = 0;
    pcre16 *p = pcre16_compile(reg, 0, &error, &error_offset, NULL);
    if (p)
    {
        m_ph = p;
        if (extra)
        {
            const char *error = NULL;
            m_pe = pcre16_study(m_ph, 0, &error);
        }
    }
    return (m_ph) ? true : false;
}

void Pcre16::find(const std::wstring& string)
{
    if (!m_ph)
        return;

    m_indexes.clear();
    int params[33];
    const unsigned short *src = (unsigned short*)string.c_str();
    int src_len = string.length();

    int count = pcre16_exec(m_ph, m_pe, src, src_len, 0, 0, params, 33);    
    for (int i=0; i<count; i++)
    {
        m_indexes.push_back(params[2*i]);
        m_indexes.push_back(params[2*i+1]);
    }
}

void Pcre16::findAllMatches(const std::wstring& string)
{
    if (!m_ph)
        return;
 
    m_indexes.clear();
    int params[33];
    const unsigned short *src = (unsigned short*)string.c_str();
    int src_len = string.length();
    int pos = 0;

    while(1)
    {
        int count = pcre16_exec(m_ph, m_pe, src, src_len, pos, 0, params, 33);
        if (count <= 0)
           break;
        m_indexes.push_back(params[0]);
        m_indexes.push_back(params[1]);
        pos = params[1];
    }
}

int Pcre16::getSize() const
{
    return m_indexes.size() / 2; 
}

int Pcre16::getFirst(int index) const
{
    assert(index>=0 && index<getSize());
    return m_indexes[index*2];
}

int Pcre16::getLast(int index) const
{
    assert(index>=0 && index<getSize());
    return m_indexes[index*2+1];
}

void Pcre16::clear()
{
    if (m_pe)
        pcre16_free_study(m_pe);
    m_pe = NULL;
    if (m_ph)
        pcre16_free(m_ph);
    m_ph = NULL;    
}

void PCRE::test()
{
    tstring t(L" %1 444 %2: %3 %45");
    tstring r(L"(%[0-9])+");

    Pcre16 p;
    p.setRegExp(r, true);
    p.findAllMatches(t);

    for (int i=0,e=p.getSize(); i<e; ++i)
    {
        char buffer[64];
        sprintf(buffer, "%d:%d \n", p.getFirst(i), p.getLast(i));
        OutputDebugStringA(buffer);        
    }
}
