#ifndef XML_HELPER_H
#define XML_HELPER_H
#include "xml.h"

namespace xml
{  
  // xml node helper
  class node
  {
  public:
      node();
      node(xnode xml_node);
      ~node();
      
      bool  load(xml::node parent_node, const char *filename);
      bool  createnew(xml::node parent_node, const char *node_name);
      bool  save(const char *filename);

      void  getname(std::string* value) const;
      bool  getattribute(const char* attribute, std::string* value) const;
      bool  getattribute(const char* attribute, int* value) const;
      void  gettext(std::string *text) const;
      void  getsubnodetext(const char* subnode, std::string* text) const;
      void  getallsubnodestext(const char* subnode, std::string* text) const;    
      void  getcdata(std::string *text) const;
      void  deletesubnodes();
      void  deletenode();

      bool  setattribute(const char* attribute, const std::string &value) const;
      bool  setattribute(const char* attribute, int value) const;
      bool  settext(const std::string &value) const;
      bool  setsubnodetext(const char* subnode, const std::string& text) const;

      xml::node createsubnode(const char* name) const;
      xml::node makecopytochild(xml::node parent_node) const;

      operator xnode() { return m_Node; }
      operator bool() const { return (m_Node) ? true : false; }

  private:
      xnode m_Node;
  };

  // xml XPath request helper
  class request
  {
  public: 
      request(const char *request_string);
      request(xml::node request_node, const char *request_string);
      ~request();

      int   size() const { return m_ListSize; }
      bool  empty() const { return (!m_ListSize); }

      const xml::node operator[](int node_index);
      void  deletenodes();

  private:
      xlist m_NodeList;
      int   m_ListSize;
  };
}

#endif // XML_HELPER_H
