#include "stdafx.h"
#include "compareObject.h"

CompareObject::CompareObject() {}
CompareObject::~CompareObject() {}

bool CompareObject::init(const tstring& key)
{
    ParamsHelper ph(key);
    if (ph.checkDoubles())
    {
        assert(false);
        return false;
    }

    tstring regexp;
    pcreEncode(key, &regexp);
    m_key = key;
    
    bool result = m_pcre.setRegExp(regexp, true);    
    assert(result);
    return result;
}

bool CompareObject::checkToCompare(const tstring& str)
    {
    m_pcre.find(str);
    if (m_pcre.getSize() == 0)
        return false;
    m_str = str;
    return true;
}

void CompareObject::replace(const tstring& newpart, tstring *result) const
{
    assert(result);
    if (m_pcre.getSize() == 0)
    {
        result->clear();
        return;
    }

    int begin =  m_pcre.getFirst(0);
    int end =  m_pcre.getLast(0);
    result->assign(m_str.substr(0, begin));
    result->append(newpart);
    result->append(m_str.substr(end));    
}

void CompareObject::getParameters(std::vector<tstring>* params) const
{
    assert(params);    
    std::vector<tstring> &p = *params;
    int size = m_pcre.getSize();
    if (size < 2)
    {
        p.clear();
        return;
    }

    ParamsHelper keys(m_key);
    int maxid = keys.getMaxId();
    p.resize(maxid+1);

    // pcre find values of %1
    for (int i=0,e=size-1; i<e; ++i)
    {
        int id = keys.getId(i);
        int begin = m_pcre.getFirst(i+1);
        int end = m_pcre.getLast(i+1);
        p[id] = m_str.substr(begin, end-begin);
    }
}

void CompareObject::pcreEncode(const tstring& src, tstring *regexp)
{
    //mask regexp special symbols
    tstring tmp;
    const WCHAR *symbols = L"*+/?[]()\\";
    const WCHAR *b = src.c_str();
    const WCHAR *e = b + src.length();

    const WCHAR* p = b + wcscspn(b, symbols);
    while (p != e)
    {
        tmp.append( tstring(b, p-b) );
        tmp.append(L"\\");
        WCHAR x[2] = { *p, 0 };
        tmp.append(x);
        b = p + 1;
        p = b + wcscspn(b, symbols);
    }
    tmp.append(b);
       
    // replace parameters, like %1 etc. to regexp part (.+?)
    int pos = 0;
    ParamsHelper ph(tmp);
    for (int i=0,e=ph.getSize(); i<e; ++i)
    {
        regexp->append( tmp.substr(pos, ph.getFirst(i)-pos) );
        regexp->append(L"(.+?)");
        pos = ph.getLast(i);       
    }
    regexp->append(tmp.substr(pos));
}

void CompareObject::getRange(CompareRange *range) const
{ 
    if (m_pcre.getSize() == 0)
        return;
    range->begin = m_pcre.getFirst(0);
    range->end = m_pcre.getLast(0); 
}
