#pragma once

class MudCommandBar :  public CWindowImpl<MudCommandBar, CEdit>
{       
    PropertiesData* propData;
    MemoryBuffer m_cmdBar;
    int m_history_index;

public:
    MudCommandBar(PropertiesData *data) : propData(data), m_history_index(-1)
    {
        initCmdBar(255);
    }

    void getCommand(tstring *cmd)
    {
        WCHAR* buffer = (WCHAR*)m_cmdBar.getData();
        cmd->assign(buffer);
        buffer[0] = 0;
    }

    void clear()
    {
        SetWindowText(L"");
    }

private:
    BEGIN_MSG_MAP(MudCommandBar)
        MESSAGE_HANDLER(WM_CHAR, OnChar)
        MESSAGE_HANDLER(WM_KEYDOWN, OnKeyDown)
    END_MSG_MAP()

    LRESULT OnChar(UINT, WPARAM wparam, LPARAM lparam, BOOL& bHandled)
    {                        
        if (wparam == 13)
        {
            int len = GetWindowTextLength();
            int buffer_len = m_cmdBar.getSize()-1;
            if (buffer_len < len)
                initCmdBar(len);
            GetWindowText((LPTSTR)m_cmdBar.getData(), m_cmdBar.getSize());
            onChangeHistory();
            SendMessage(GetParent(), WM_USER, 0, 0);
            return 0;
        }
        bHandled = FALSE;
        return 0;
    }

    LRESULT OnKeyDown(UINT, WPARAM wparam, LPARAM, BOOL& bHandled)
    {
        if (wparam == VK_DOWN)
            onHistoryDown();
        else if (wparam == VK_UP)        
            onHistoryUp();
        else
            bHandled = FALSE;
        return 0;
    }

    void initCmdBar(int size)
    {
        m_cmdBar.alloc((size+1) * sizeof(WCHAR));
    }

    void onHistoryUp()
    {
        std::vector<tstring> &h = propData->cmd_history;
        if (h.empty())
            return;

        if (m_history_index == -1)
            m_history_index = h.size() - 1;
        else
        {
            if (m_history_index != 0)
                m_history_index -= 1;
        }
        const tstring& cmd = h[m_history_index];
        SetWindowText(cmd.c_str());
        int lastsym = cmd.length();
        SetSel(lastsym, lastsym);
    }

    void onHistoryDown()
    {
        if (m_history_index == -1)
            return;
        std::vector<tstring> &h = propData->cmd_history;
        int last_history = h.size() -1;
        if (m_history_index != last_history)
            m_history_index += 1;
        else
        {
            clear();
            m_history_index = -1;
            return;
        }
        const tstring& cmd = h[m_history_index];
        SetWindowText(cmd.c_str());
        int lastsym = cmd.length();
        SetSel(lastsym, lastsym);
    }

    void onChangeHistory()
    {
        WCHAR* buffer = (WCHAR*)m_cmdBar.getData();
        tstring cmd(buffer);
        if (cmd.empty())
            return;

        std::vector<tstring> &h = propData->cmd_history;
        if (m_history_index != -1)        
        {
            if (h[m_history_index] == cmd)            
                h.erase(h.begin() + m_history_index);
            m_history_index = -1;
        }

        if (!h.empty())
        {
            int last = h.size() - 1;
            if (h[last] == cmd)
                return;
        }
        h.push_back(cmd);

        int size = h.size();
        if (size > propData->cmd_history_size)
        {
            size = size - propData->cmd_history_size;
            h.erase(h.begin(), h.begin()+size);
        }
    }
};
