#pragma once

#include "propertiesPages/propertiesDlg.h"
#include "propertiesPages/propertiesManager.h"
#include "propertiesPages/profileDlgs.h"

#include "network/network.h"
#include "mudViewParser.h"
#include "mudView.h"
#include "mudCommandBar.h"
#include "systemInterpretator.h"
#include "incomingProcessor.h"
#include "AboutDlg.h"

class MudGameView : public CWindowImpl<MudGameView>, public SystemInterpretatorCallback
{
    PropertiesManager m_manager;
    PropertiesElements m_propElements;
    PropertiesElements *propElements;
    PropertiesData *propData;

    CDockingWindow m_dock;
    
    int m_barHeigth;
    MudCommandBar m_bar;
    MudView m_view;

    MudViewParser m_parser;
    IncomingProcessor m_incomingProcessor;
    Network m_network;
    HotkeyTable m_hotkeyTable;
    SystemInterpretator m_syscmds;    
    AnsiToWide m_convertorA2W;
    WideToAnsi m_convertorW2A;

    MudView* m_views[4];

private:
    bool onStart();
    void onClose();
    void onNewProfile();
    void onLoadProfile();
    void onNewCommandBarCmd(tstring cmd);
    void onNewNetworkData();
    void onDisconnectNetwork();
    bool onProcessHotkey(const tstring& key);
    void onProcessSytemCommand(const tstring& cmd);
    void processIncoming(const WCHAR* text, int text_len, bool skip_actions = false);

public:
    MudGameView() : m_propElements(m_manager.getConfig()), propElements(&m_propElements), propData(m_propElements.propData),
        m_bar(propData), m_view(propElements), m_incomingProcessor(propData),
        m_barHeigth(16), m_connected(false)

    {
    }

    DECLARE_WND_CLASS(NULL)
    BOOL PreTranslateMessage(MSG* pMsg)
    {
        if (pMsg->message == WM_KEYDOWN || pMsg->message == WM_SYSKEYDOWN)
        {
            tstring key;
            m_hotkeyTable.recognize(pMsg->wParam, pMsg->lParam, &key);
            if (onProcessHotkey(key))
                return TRUE;
        }
		return FALSE;
	}

    HWND createView(HWND parent)
    {
        HWND dock = m_dock.Create(parent, rcDefault);
        Create(dock, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);
        m_dock.SetClient(m_hWnd);
        return dock;
    }

    void addOutput()
    {
        MudView *v0 = new MudView(propElements);
        v0->Create(m_dock, rcDefault, _T("View1"),  WS_CHILD|WS_VISIBLE|WS_CLIPSIBLINGS|WS_CLIPCHILDREN, WS_EX_CLIENTEDGE);
        m_dock.AddWindow(*v0);
        m_dock.DockWindow(*v0, DOCK_RIGHT);

        MudView *v1 = new MudView(propElements);
        v1->Create(m_dock, rcDefault, _T("View1"),  WS_CHILD|WS_VISIBLE|WS_CLIPSIBLINGS|WS_CLIPCHILDREN, WS_EX_CLIENTEDGE);
        m_dock.AddWindow(*v1);
        m_dock.DockWindow(*v1, DOCK_LEFT);
    }

private:
    BEGIN_MSG_MAP(MudGameView)
        MESSAGE_HANDLER(WM_CREATE, OnCreate)
        MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
        MESSAGE_HANDLER(WM_SIZE, OnSize)
        MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBkgnd)
        MESSAGE_HANDLER(WM_USER, OnUserCommand)
        MESSAGE_HANDLER(WM_USER+1, OnNetwork)
    ALT_MSG_MAP(1)  // retranslated from MainFrame
        MESSAGE_HANDLER(WM_SETFOCUS, OnSetFocus)
        MESSAGE_HANDLER(WM_MOUSEWHEEL, OnWheel)
        COMMAND_ID_HANDLER(ID_FILE_NEW, OnNewProfile)
        COMMAND_ID_HANDLER(ID_FILE_OPEN, OnLoadProfile)
        COMMAND_ID_HANDLER(ID_SETTINGS, OnSettings)
        COMMAND_ID_HANDLER(ID_WINDOW_1, OnShowWindow)
        COMMAND_ID_HANDLER(ID_WINDOW_2, OnShowWindow)
        COMMAND_ID_HANDLER(ID_WINDOW_3, OnShowWindow)
        COMMAND_ID_HANDLER(ID_WINDOW_4, OnShowWindow)
	END_MSG_MAP()

    LRESULT OnCreate(UINT, WPARAM, LPARAM lparam, BOOL& bHandled)
	{
        m_view.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VSCROLL, WS_EX_CLIENTEDGE);
        m_bar.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, WS_EX_CLIENTEDGE);
        if (!onStart())
            ::PostMessage(GetParent(), WM_CLOSE, 0, 0);

        addOutput();
        return 0;
    }

    LRESULT OnDestroy(UINT, WPARAM, LPARAM lparam, BOOL&)
	{
        onClose();
        return 0;
    }
    
    LRESULT OnSize(UINT, WPARAM, LPARAM lparam, BOOL& bHandled)
	{
        if (!m_view.IsWindow())
            return 0;
        int width = LOWORD(lparam);
        int heigth = HIWORD(lparam);
        RECT pos = { 0, 0, width, heigth - m_barHeigth };
        m_view.MoveWindow(&pos);
        pos.top = pos.bottom;
        pos.bottom = heigth;
        m_bar.MoveWindow(&pos);
        return 0;
    }

    LRESULT OnEraseBkgnd(UINT, WPARAM, LPARAM lparam, BOOL&)
    {
        return 1;
    }

    LRESULT OnWheel(UINT, WPARAM wparam, LPARAM lparam, BOOL&)
    {
        if (m_view.IsWindow())
            ::SendMessage(m_view, WM_MOUSEWHEEL, wparam, lparam);
        return 0;
    }

    LRESULT OnSetFocus(UINT, WPARAM, LPARAM lparam, BOOL& bHandled)
    {
        if (m_bar.IsWindow())
            m_bar.SetFocus();
        return 0;
    }

    LRESULT OnNetwork(UINT, WPARAM, LPARAM lparam, BOOL&)
    {
        NetworkEvents result = m_network.processMsg(lparam);
        if (result == NE_NEWDATA)
        {
            onNewNetworkData();            
        }
        else if (result == NE_DISCONNECT)
        {
            onDisconnectNetwork();
        }
        return 0;
    }
   
    LRESULT OnUserCommand(UINT, WPARAM wparam, LPARAM, BOOL&)
    {
        tstring cmd;
        m_bar.getCommand(&cmd);        
        m_bar.clear();
        onNewCommandBarCmd(cmd);
		return 0;
    }

    void initCommandBar()
    {
        m_barHeigth = propElements->font_height + 4;
        m_bar.SetFont(propElements->standard_font);
    }

    LRESULT OnNewProfile(WORD, WORD, HWND, BOOL&)
    {
        onNewProfile();        
        return 0;
    }

    LRESULT OnLoadProfile(WORD, WORD, HWND, BOOL&)
    {
        onLoadProfile();        
        return 0;
    }  

    LRESULT OnSettings(WORD, WORD, HWND, BOOL&)
    {
        PropertiesData& data = *m_manager.getConfig();
        PropertiesData tmp(data);
        PropertiesDlg propDlg(&tmp);
        if (propDlg.DoModal() == IDOK)
        {
            data = tmp;
            updateProps();
            if (!m_manager.saveProfile())
                msgBox(m_hWnd, IDS_ERROR_SAVEPROFILE_FAILED, MB_OK|MB_ICONSTOP);
        }
        return 0;
    }

    
    LRESULT OnShowWindow(WORD, WORD, HWND, BOOL&)
    {
        return 0;
    }
        
    void updateProps()
    {
       m_propElements.updateProps(m_hWnd);
       initCommandBar();
       m_view.updateProps();
       m_incomingProcessor.updateProps();
       updateTitle();
    }

    void updateTitle()
    {
        tstring title;
        title.append(m_manager.getProfileGroup());
        title.append(L" - ");
        title.append(m_manager.getProfileName());
        title.append(L" - ");
        tstring appname;
        loadString(IDR_MAINFRAME, &appname);
        title.append(appname);     
        ::SetWindowText(m_dock.GetParent(), title.c_str());
    }

public:
    void test()
    {
        BOOL b;
        OnSettings(0,0,0,b);    
    }    

private:
    bool m_connected;
    void siLog(const tstring& cmd);
    void siError(const tstring& cmd, const tstring& error);    
    void siConnect(const tstring& address, int port);    
    void siLostConnection();
};
