#include "stdafx.h"
#include "network.h"
#include <winsock2.h>
#pragma comment(lib, "ws2_32.lib")

#define IAC             255
#define DONT            254
#define DO              253
#define WONT            252
#define WILL            251
#define SB              250
#define SE              240
#define COMPRESS        85
#define COMPRESS2       86

Network::Network() : m_can_send(false)
{
    m_input_buffer.alloc(1024);
}

Network::~Network()
{
    close();
}

bool Network::connect(const NetworkConnectData& data)
{
    sock = WSASocket(AF_INET, SOCK_STREAM, IPPROTO_TCP, NULL, 0, 0);
    if (sock == INVALID_SOCKET)
        return false;

    sockaddr_in peer; 
    memset(&peer, 0, sizeof(sockaddr_in));
    peer.sin_family = AF_INET;
    peer.sin_port = htons( data.port );

    const std::string& a = data.address;
    if ( strspn( a.c_str(), "0123456789." ) != a.length() )
    {
        struct hostent *hp = gethostbyname(a.c_str());
        if (!hp)
            return false;
        memcpy((char*)&peer.sin_addr, hp->h_addr, sizeof(peer.sin_addr));
    }
    else
    {       
       peer.sin_addr.s_addr = inet_addr( data.address.c_str() );
    }

    if (WSAConnect( sock, (SOCKADDR*)&peer, sizeof(peer), NULL, NULL, NULL, NULL) == SOCKET_ERROR) 
    {
       close();
       return false;    
    }       

    if (WSAAsyncSelect(sock, data.wndToNotify, data.notifyMsg, FD_READ|FD_WRITE|FD_CONNECT|FD_CLOSE) == SOCKET_ERROR)
    {
        close();
        return false;
    }
    return true;
}

void Network::close()
{
    closesocket(sock);
    sock = NULL;
}

NetworkEvents Network::processMsg(DWORD msg_lparam)
{
    WORD error = WSAGETSELECTERROR(msg_lparam);
    if (error)    
        return NE_ERROR;

    WORD event = WSAGETSELECTEVENT(msg_lparam);
    if (event == FD_READ)
    {
        int result = read_socket();
        if (result == -1)
            return NE_ERROR;
        return (result != 0) ? NE_NEWDATA : NE_NOEVENT;
    }
    else if (event == FD_WRITE)
    {
        return (write_socket() == -1) ? NE_ERROR : NE_NOEVENT;
    }
    else if (event == FD_CLOSE)
    {
        return NE_DISCONNECT;
    }
    return NE_NOEVENT;
}

int Network::send(const tbyte* data, int len)
{
    const tbyte *b = data;
    const tbyte *e = b + len;

    for (const tbyte *p=b; p!=e; ++p)
    {
        if (*p == IAC)
        {
            m_output_buffer.write(b, (p-b)+1);
            tbyte iac = IAC;
            m_output_buffer.write(&iac, 1);
            b = p+1;
        }
    }
    if (b != e)
        m_output_buffer.write(b, e-b);

    int data_len = m_output_buffer.getSize();
    bool result = send_ex((tbyte*)m_output_buffer.getData(), data_len);
    m_output_buffer.truncate(data_len);
    return result ? len : -1;
}

DataQueue* Network::receive()
{
    return &m_receive_data;
}

int Network::read_socket()
{
    WSABUF buffer;
    buffer.buf = m_input_buffer.getData();
    buffer.len = m_input_buffer.getSize();

    DWORD flags = 0;
    DWORD readed = 0;
    if (WSARecv(sock, &buffer, 1, &readed, &flags, NULL, NULL) == SOCKET_ERROR)    
        return -1;
    if (readed == 0)
        return 0;

    m_input_data.write(m_input_buffer.getData(), readed);
    while(m_input_data.getSize() > 0)
    {
        const tbyte* in = (tbyte*)m_input_data.getData();
        int in_len = m_input_data.getSize();

        bool error = false;
        int processed = processing_data(in, in_len, &error);
        if (error)
            return -1;

        if (processed == 0)      // low data for processing
        {
            break;
        }
        else if (processed < 0)  // truncated system data
        {
            processed = -processed;
        }
        else
        {
            if (processed == 2 && in[0] == IAC && in[1] == IAC)
                m_receive_data.write(in, 1);
            else
                m_receive_data.write(in, processed);
        }
        m_input_data.truncate(processed);        
    }

    return 1;
}

int Network::write_socket()
{
    if (m_send_data.getSize() == 0)
    {
        // m_can_send = true;
         return 0;
    }    

    m_can_send = false;

    WSABUF buffer;
    buffer.buf = (char*)m_send_data.getData();
    buffer.len = m_send_data.getSize();

    DWORD flags = 0;
    DWORD sent = 0;
    if (WSASend(sock, &buffer, 1, &sent, flags, NULL, NULL) == SOCKET_ERROR)
        return -1;
    m_send_data.truncate(sent);
    return sent;
}

bool Network::send_ex(const tbyte* data, int len)
{
    assert(len >= 0);
    if (len < 0)
        return false;
    if (len == 0)
        return true;
       
    //if (!m_can_send)
      //  return true;
    m_send_data.write(data, len);

    int sent = write_socket();
    if (sent == -1)
        return false;
    return true;
}

int Network::processing_data(const tbyte* buffer, int len, bool *error)
{
    const tbyte* b = buffer;
    const tbyte* e = b;
    if (*b != IAC)                // find iac symbol
    {
        const tbyte* e = b + len;
        while(b != e)
        {
            if (*b == IAC)
                break;
            b++;
        }
        return len-(e-b);
    }

    if (len < 2)
        return 0;

    e++;
    if (*e == IAC)               // double iac - continue
        return 2;

    if (len < 3)
        return 0;

    if (e[0] == WILL)
    {
        if (e[1] == COMPRESS || e[1] == COMPRESS2)
        {
            // server send request for enable MCCP
            tbyte support[3] = { IAC, DONT, e[1] }; //todo temp off mccp
            if (!send_ex(support, 3))
               { *error = true; return 0; }
        }
        else
        {
            // report, what client don't support other Telnet options
            tbyte not_support[3] = { IAC, DONT, e[1] };
            if (!send_ex(not_support, 3))
                { *error = true; return 0; }
        }
        return -3;
    }

    if (len < 5)
        return 0;

    if (e[0] == SB && (e[1] == COMPRESS || e[1] == COMPRESS2) && e[3] == SE)
    {
        if ((e[2] == IAC && e[1] == COMPRESS2) || (e[2] == WILL && e[1] == COMPRESS))
            turnon_mccp();
        return -5;
    }
    
    return -len;    // skip error data
}

void Network::turnon_mccp()
{

}

