#pragma once

#define DEFAULT_CMD_HISTORY_SIZE 20
#define DEFAULT_VIEW_HISTORY_SIZE 400
#define MIN_CMD_HISTORY_SIZE 10
#define MIN_VIEW_HISTORY_SIZE 100
#define MAX_CMD_HISTORY_SIZE 30
#define MAX_VIEW_HISTORY_SIZE 999

template <class T, class V>
class PropertiesType
{
public:
    struct el {
        T key;
        V value;
    };

    int find(const T& key)
    {
        for (int i=0,e=m_values.size(); i<e; ++i)
            if (m_values[i].key == key) { return i; } 
        return -1;
    }

    void add(int index, const T& key, const V& value)
    {
        el data; data.key = key; data.value = value;
        if (index == -1)
            m_values.push_back(data);
        else
        {
            int size = m_values.size();
            if (index >= 0 && index < size)
                m_values[index] = data;
            else
                { assert(false); }
        }
    }
    void del(int index)
    {
        int size = m_values.size();
        if (index >= 0 && index < size)        
            m_values.erase(m_values.begin() + index);        
        else
           { assert(false); }
    }

    void swap(int index1, int index2)
    {
        int size = m_values.size();
        if (index1 >= 0 && index1 < size && index2 >= 0 && index2 < size)
        {
            if (index1 != index2)
            {
                el t = m_values[index1];
                m_values[index1] = m_values[index2];
                m_values[index2] = t;
            }
        }
    }

    const el& get(int index) const
    {
        return m_values[index];
    }

    int size() const
    {
        return m_values.size();
    }
    
    void clear()
    {
        m_values.clear();
    }

    bool exist(const T& key)
    {
        return (find(key) == -1) ? false : true;
    }


private:
    std::vector<el> m_values;
};

struct PropertiesHighlight
{
    PropertiesHighlight() : textcolor(RGB(192,192,192)), bkgcolor(RGB(0,0,0)),
        underlined(0), border(0), italic(0) {}
    COLORREF textcolor;
    COLORREF bkgcolor;
    int underlined;
    int border;
    int italic;
};

struct PropertiesData
{
    PropertiesData() : cmd_separator(L';'), cmd_prefix(L'#'), par_prefix(L'%'), 
        view_history_size(DEFAULT_VIEW_HISTORY_SIZE), cmd_history_size(DEFAULT_CMD_HISTORY_SIZE)
    {
        initDefaultColorsAndFont(); 
    }

    PropertiesType<tstring, tstring> aliases;
    typedef PropertiesType<tstring, tstring>::el alias_data;

    PropertiesType<tstring, tstring> actions;        
    typedef PropertiesType<tstring, tstring>::el action_data;

    PropertiesType<tstring, tstring> subs;
    typedef PropertiesType<tstring, tstring>::el sub_data;

    PropertiesType<tstring, tstring> hotkeys;
    typedef PropertiesType<tstring, tstring>::el hotkey_data;

    PropertiesType<tstring, PropertiesHighlight> highlights;
    typedef PropertiesType<tstring, PropertiesHighlight>::el highlight_data;

    std::vector<tstring> cmd_history;

    COLORREF colors[16];
    COLORREF bkgnd;
    tstring  font_name;
    int      font_heigth;
    int      font_bold;
    int      font_italic;

    WCHAR    cmd_separator;
    WCHAR    cmd_prefix;
    WCHAR    par_prefix;

    int      view_history_size;
    int      cmd_history_size;

    void initDefaultColorsAndFont()
    {
        bkgnd = RGB(0, 0, 0);
        colors[0] = RGB(64, 64, 64); 
        colors[1] = RGB(128, 0, 0);
        colors[2] = RGB(0, 128, 0);
        colors[3] = RGB(128, 128, 0);
        colors[4] = RGB(0, 0, 128);
        colors[5] = RGB(128, 0, 128);
        colors[6] = RGB(0, 128, 128);
        colors[7] = RGB(192, 192, 192);
        colors[8] = RGB(128, 128, 128); 
        colors[9] = RGB(255, 0, 0);
        colors[10] = RGB(0, 255, 0);
        colors[11] = RGB(255, 255, 0);
        colors[12] = RGB(0, 0, 255);
        colors[13] = RGB(255, 0, 255);
        colors[14] = RGB(0, 255, 255);
        colors[15] = RGB(255, 255, 255);

        font_name.assign(L"Fixedsys");
        font_heigth = 10;
        font_bold = FW_NORMAL;
        font_italic = 0;
    }

    void initAllDefault()
    {
        aliases.clear();
        actions.clear();
        subs.clear();
        hotkeys.clear();
        highlights.clear();
        cmd_history.clear();
        initDefaultColorsAndFont();
    }

    void initLogFont(HWND hwnd, LOGFONT *f)
    {
        f->lfHeight = -MulDiv(font_heigth, GetDeviceCaps(GetDC(hwnd), LOGPIXELSY), 72);
        f->lfWidth = 0;
        f->lfEscapement = 0;
        f->lfOrientation = 0;        
        f->lfWeight = font_bold;
        f->lfItalic = font_italic ? 1 : 0;
        f->lfUnderline = 0;
        f->lfStrikeOut = 0;
        f->lfCharSet = DEFAULT_CHARSET;
        f->lfOutPrecision = OUT_DEFAULT_PRECIS;
        f->lfClipPrecision = CLIP_DEFAULT_PRECIS;
        f->lfQuality = DEFAULT_QUALITY;
        f->lfPitchAndFamily = DEFAULT_PITCH;
        wcscpy(f->lfFaceName, font_name.c_str() );
    }
};

