#pragma once

class PropertyAliases :  public CDialogImpl<PropertyAliases>
{
    PropertiesData *propData;
    PropertyListCtrl m_list;
    CBevelLine m_bl1;
    CBevelLine m_bl2;
    CEdit m_alias;
    CEdit m_text;
    CButton m_add;
    CButton m_del;
    bool m_block;

public:
     enum { IDD = IDD_PROPERTY_ALIASES };
     PropertyAliases(PropertiesData *data) : propData(data), m_block(false) {}

private:
    BEGIN_MSG_MAP(PropertyAliases)
       MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
       COMMAND_ID_HANDLER(IDC_BUTTON_ALIAS_ADD, OnAddElement)
       COMMAND_ID_HANDLER(IDC_BUTTON_ALIAS_DEL, OnDeleteElement)
       COMMAND_HANDLER(IDC_EDIT_ALIAS, EN_CHANGE, OnAliasEditChanged)
       NOTIFY_HANDLER(IDC_LIST_ALIASES, LVN_ITEMCHANGED, OnListItemChanged)
       NOTIFY_HANDLER(IDC_LIST_ALIASES, NM_SETFOCUS, OnListItemChanged)
       NOTIFY_HANDLER(IDC_LIST_ALIASES, NM_KILLFOCUS, OnListKillFocus)
    END_MSG_MAP()

    LRESULT OnAddElement(WORD, WORD, HWND, BOOL&)
    {
        tstring alias, text;
        getWindowText(m_alias, &alias);
        getWindowText(m_text, &text);        
        int index = propData->aliases.find(alias);
        if (index == -1)
        {            
            int pos = m_list.GetItemCount();
            m_list.AddItem(pos, 0, alias.c_str());
            m_list.AddItem(pos, 1, text.c_str());
        }
        else
        {
            m_block = true;
            m_list.SetItem(index, 0, LVIF_TEXT, alias.c_str(), 0, 0, 0, NULL);
            m_list.SetItem(index, 1, LVIF_TEXT, text.c_str(), 0, 0, 0, NULL);
        }
        propData->aliases.add(index, alias, text);
        if (index == -1)
            index = m_list.GetItemCount()-1;
        m_list.SelectItem(index);
        return 0;
    }

    LRESULT OnDeleteElement(WORD, WORD, HWND, BOOL&)
    {   
        int index = m_list.GetSelectedIndex();
        m_list.DeleteItem(index);        
        propData->aliases.del(index);
        return 0;
    }

    LRESULT OnAliasEditChanged(WORD, WORD, HWND, BOOL&)
    {
        int string_len = m_alias.GetWindowTextLength();        
        m_add.EnableWindow(string_len == 0 ? FALSE : TRUE);
        return 0;
    }

    LRESULT OnListItemChanged(int , LPNMHDR , BOOL&)
    {
        int item_selected = m_list.GetSelectedIndex();
        if (item_selected == -1)
            m_del.EnableWindow(FALSE);
        else
            m_del.EnableWindow(TRUE);

        if (item_selected != -1 && !m_block)
        {
            const PropertiesData::alias_data& alias = propData->aliases.get(item_selected);
            m_alias.SetWindowText(alias.key.c_str() );
            m_text.SetWindowText(alias.value.c_str());
        }
        m_block = false;
        return 0;
    }    

    LRESULT OnListKillFocus(int , LPNMHDR , BOOL&)
    {
        if (GetFocus() != m_del)        
            m_del.EnableWindow(FALSE);            
        return 0;
    }
    
    LRESULT OnInitDialog(UINT, WPARAM, LPARAM, BOOL&)
	{
        m_alias.Attach(GetDlgItem(IDC_EDIT_ALIAS));
        m_text.Attach(GetDlgItem(IDC_EDIT_ALIAS_TEXT));
        m_add.Attach(GetDlgItem(IDC_BUTTON_ALIAS_ADD));
        m_del.Attach(GetDlgItem(IDC_BUTTON_ALIAS_DEL));
        m_list.Attach(GetDlgItem(IDC_LIST_ALIASES));
        m_list.addColumn(L"Alias", 50);
        m_list.addColumn(L"", 50);
        m_list.SetExtendedListViewStyle( m_list.GetExtendedListViewStyle() | LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT);
        m_bl1.SubclassWindow(GetDlgItem(IDC_STATIC_ALBL1));
        m_bl2.SubclassWindow(GetDlgItem(IDC_STATIC_ALBL2));
        m_add.EnableWindow(FALSE);
        m_del.EnableWindow(FALSE);

        for (int i=0,e=propData->aliases.size(); i<e; ++i)
        {
            const PropertiesData::alias_data& alias = propData->aliases.get(i);
            m_list.AddItem(i, 0, alias.key.c_str());
            m_list.AddItem(i, 1, alias.value.c_str());
        }
        return 0;
    }
};
