#pragma once
#include "propertiesPagesElements.h"

class PropertyHighlights :  public CDialogImpl<PropertyHighlights>, public PropertyListCtrlHandler
{
    PropertiesData *propData;
    PropertyListCtrl m_list;
    CBevelLine m_bl1;
    CBevelLine m_bl2;
    CEdit m_pattern;
    CButton m_add;
    CButton m_del;
    bool m_block;

    HighlightSelectColor m_textColor;
    HighlightSelectColor m_bkgColor;
    HighlightsExampleWindow m_exampleWnd;

    CButton m_underline;
    CButton m_border;
    CButton m_italic;

public:
     enum { IDD = IDD_PROPERTY_HIGHLIGHTS };
     PropertyHighlights(PropertiesData *data) : propData(data), m_block(false),
         m_textColor(RGB(192,192,192), WM_USER), m_bkgColor(RGB(0,0,0), WM_USER+1),
     m_exampleWnd(data) {}

private:
    BEGIN_MSG_MAP(PropertyHighlights)
       MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
       MESSAGE_HANDLER(WM_SHOWWINDOW, OnShowWindow)
       MESSAGE_HANDLER(WM_USER, OnTextColor)
       MESSAGE_HANDLER(WM_USER+1, OnBkgColor)
       COMMAND_ID_HANDLER(IDC_CHECK_HIGHLIGHTS_UNDERLINE, OnFontUnderlined)
       COMMAND_ID_HANDLER(IDC_CHECK_HIGHLIGHTS_FLASH, OnFontBorder)
       COMMAND_ID_HANDLER(IDC_CHECK_HIGHLIGHTS_ITALIC, OnFontItalic)
       COMMAND_ID_HANDLER(IDC_BUTTON_HIGHLIGHT_ADD, OnAddElement)
       COMMAND_ID_HANDLER(IDC_BUTTON_HIGHLIGHT_DEL, OnDeleteElement)
       COMMAND_HANDLER(IDC_EDIT_HIGHLIGHT_TEXT, EN_CHANGE, OnPatternEditChanged)
       NOTIFY_HANDLER(IDC_LIST_HIGHLIGHTS, LVN_ITEMCHANGED, OnListItemChanged)
       NOTIFY_HANDLER(IDC_LIST_HIGHLIGHTS, NM_SETFOCUS, OnListItemChanged)
       NOTIFY_HANDLER(IDC_LIST_HIGHLIGHTS, NM_KILLFOCUS, OnListKillFocus)
       REFLECT_NOTIFICATIONS()
    END_MSG_MAP()

    // handler for draw item
    void drawPropertyListItem(NMLVCUSTOMDRAW *pLVCD)
    {
        int subitem = pLVCD->iSubItem;
        if (subitem != 2 && subitem != 3) // 2 - text color, 3 - background color
            return;

        int item = pLVCD->nmcd.dwItemSpec;
        
        const PropertiesData::highlight_data& hd = propData->highlights.get(item);
        const PropertiesHighlight& hl = hd.value;
        pLVCD->clrTextBk = (subitem == 2) ? hl.textcolor : hl.bkgcolor;    
    }

    LRESULT OnAddElement(WORD, WORD, HWND, BOOL&)
    {
        tstring pattern;
        getWindowText(m_pattern, &pattern);

        PropertiesHighlight hl;
        hl.textcolor = m_textColor.getColor();
        hl.bkgcolor = m_bkgColor.getColor();
        hl.underlined = m_underline.GetCheck() ? 1 : 0;
        hl.border = m_border.GetCheck() ? 1 : 0;
        hl.italic = m_italic.GetCheck() ? 1 : 0;
        
        tstring flags;
        if (hl.underlined) flags.append(L"");        
        if (hl.border) flags.append(L"");
        if (hl.italic) flags.append(L"");
        if (flags.empty())
            flags.append(L"-");

        int index = propData->highlights.find(pattern);
        if (index == -1)
        {            
            int pos = m_list.GetItemCount();
            m_list.AddItem(pos, 0, pattern.c_str());
            m_list.AddItem(pos, 1, flags.c_str());
        }
        else
        {
            m_block = true;
            m_list.SetItem(index, 0, LVIF_TEXT, pattern.c_str(), 0, 0, 0, NULL);
            m_list.SetItem(index, 1, LVIF_TEXT, flags.c_str(), 0, 0, 0, NULL);
        }

        propData->highlights.add(index, pattern, hl);
        if (index == -1)
            index = m_list.GetItemCount()-1;
        m_list.SelectItem(index);
        return 0;
    }

    LRESULT OnDeleteElement(WORD, WORD, HWND, BOOL&)
    {   
        int index = m_list.GetSelectedIndex();
        m_list.DeleteItem(index);        
        propData->highlights.del(index);
        return 0;
    }

    LRESULT OnFontUnderlined(WORD, WORD, HWND, BOOL&)
    {
        bool checked = m_underline.GetCheck() ? true : false;
        m_exampleWnd.setUnderlined(checked);
        return 0;
    }

    LRESULT OnFontBorder(WORD, WORD, HWND, BOOL&)
    {
        bool checked = m_border.GetCheck() ? true : false;
        m_exampleWnd.setBorder(checked);
        return 0;
    }

    LRESULT OnFontItalic(WORD, WORD, HWND, BOOL&)
    {
        bool checked = m_italic.GetCheck() ? true : false;
        m_exampleWnd.setItalic(checked);
        return 0;
    }

    LRESULT OnPatternEditChanged(WORD, WORD, HWND, BOOL&)
    {
        int string_len = m_pattern.GetWindowTextLength();        
        m_add.EnableWindow(string_len == 0 ? FALSE : TRUE);
        return 0;
    }

    LRESULT OnListItemChanged(int , LPNMHDR , BOOL&)
    {
        int item_selected = m_list.GetSelectedIndex();
        if (item_selected == -1)
            m_del.EnableWindow(FALSE);
        else
            m_del.EnableWindow(TRUE);

        if (item_selected != -1 && !m_block)
        {
            const PropertiesData::highlight_data& hd = propData->highlights.get(item_selected);
            const PropertiesHighlight& hl = hd.value;

            m_pattern.SetWindowText( hd.key.c_str() );
            m_textColor.setColor(hl.textcolor);
            m_bkgColor.setColor(hl.bkgcolor);
            m_underline.SetCheck(hl.underlined ? BST_CHECKED : BST_UNCHECKED);
            m_border.SetCheck(hl.border ? BST_CHECKED : BST_UNCHECKED);
            m_italic.SetCheck(hl.italic ? BST_CHECKED : BST_UNCHECKED);
            m_exampleWnd.setAllParameters(hl.textcolor, hl.bkgcolor, 
                hl.underlined ? true:false, 
                hl.border ? true:false,
                hl.italic ? true:false);
        }
        m_block = false;
        return 0;
    }    

    LRESULT OnListKillFocus(int , LPNMHDR , BOOL&)
    {
        if (GetFocus() != m_del)        
            m_del.EnableWindow(FALSE);
        return 0;
    }
    
    LRESULT OnInitDialog(UINT, WPARAM, LPARAM, BOOL&)
	{
        m_pattern.Attach(GetDlgItem(IDC_EDIT_HIGHLIGHT_TEXT));

        RECT rc;
        CStatic tc(GetDlgItem(IDC_STATIC_TEXTCOLOR));
        tc.GetWindowRect(&rc);
        tc.ShowWindow(SW_HIDE);
        ScreenToClient(&rc);
        m_textColor.Create(m_hWnd, rc, NULL, WS_CHILD|WS_VISIBLE|WS_TABSTOP);
        
        CStatic bc(GetDlgItem(IDC_STATIC_BKGCOLOR));
        bc.GetWindowRect(&rc);
        bc.ShowWindow(SW_HIDE);
        ScreenToClient(&rc);
        m_bkgColor.Create(m_hWnd, rc, NULL, WS_CHILD|WS_VISIBLE|WS_TABSTOP);

        CStatic he(GetDlgItem(IDC_STATIC_HIGHLIGHTS_EXAMPLE));
        he.GetWindowRect(&rc);
        he.ShowWindow(SW_HIDE);
        ScreenToClient(&rc);
        m_exampleWnd.Create(m_hWnd, rc, NULL, WS_CHILD|WS_VISIBLE);
               
        m_add.Attach(GetDlgItem(IDC_BUTTON_HIGHLIGHT_ADD));
        m_del.Attach(GetDlgItem(IDC_BUTTON_HIGHLIGHT_DEL));
        m_list.Attach(GetDlgItem(IDC_LIST_HIGHLIGHTS));
        m_list.setHandler(this);
        m_list.addColumn(L"", 50);
        m_list.addColumn(L"", 10);
        m_list.addColumn(L" ", 20);
        m_list.addColumn(L" ", 20);        
        m_list.SetExtendedListViewStyle( m_list.GetExtendedListViewStyle() | LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT);        

        m_bl1.SubclassWindow(GetDlgItem(IDC_STATIC_HL_BL1)); 
        m_bl2.SubclassWindow(GetDlgItem(IDC_STATIC_HL_BL2));
        m_add.EnableWindow(FALSE);
        m_del.EnableWindow(FALSE);

        m_underline.Attach(GetDlgItem(IDC_CHECK_HIGHLIGHTS_UNDERLINE));
        m_border.Attach(GetDlgItem(IDC_CHECK_HIGHLIGHTS_FLASH));
        m_italic.Attach(GetDlgItem(IDC_CHECK_HIGHLIGHTS_ITALIC));

        for (int i=0,e=propData->highlights.size(); i<e; ++i)
        {
            const PropertiesData::highlight_data& hd = propData->highlights.get(i);
            const PropertiesHighlight& hl = hd.value;
            
            tstring flags;
            if (hl.underlined) flags.append(L"");        
            if (hl.border) flags.append(L"");
            if (hl.italic) flags.append(L"");
            if (flags.empty())
                flags.append(L"-");

            m_list.AddItem(i, 0, hd.key.c_str());
            m_list.AddItem(i, 1, flags.c_str());

        }
        return 0;
    }
  
    LRESULT OnShowWindow(UINT, WPARAM wparam, LPARAM, BOOL&)
    {
        if (wparam)        
            m_exampleWnd.updateProps();
        return 0;
    }    

    LRESULT OnTextColor(UINT, WPARAM, LPARAM, BOOL&)
    {
        COLORREF color = m_textColor.getColor();
        CColorDialog dlg(color, CC_FULLOPEN, m_hWnd);
        if (dlg.DoModal() == IDOK)
        {
            color = dlg.GetColor();
            m_textColor.setColor(color);
            m_exampleWnd.setTextColor(color);
        }

        return 0;
    }

    LRESULT OnBkgColor(UINT, WPARAM, LPARAM, BOOL&)
    {
        COLORREF color = m_bkgColor.getColor();
        CColorDialog dlg(color, CC_FULLOPEN, m_hWnd);
        if (dlg.DoModal() == IDOK)
        {
            color = dlg.GetColor();
            m_bkgColor.setColor(color);
            m_exampleWnd.setBkgColor(color);
        }

        return 0;
    }
};
