#pragma once

#include "hotkeyTable.h"

class HotkeyBox :  public CWindowImpl<HotkeyBox, CEdit>
{
    HotkeyTable m_table;
  
    BEGIN_MSG_MAP(HotkeyBox)
       MESSAGE_HANDLER(WM_KEYUP, OnKeyUp)
       MESSAGE_HANDLER(WM_SYSKEYUP, OnKeyUp)
       MESSAGE_HANDLER(WM_KEYDOWN, OnBlock)
       MESSAGE_HANDLER(WM_SYSKEYDOWN, OnBlock)
       MESSAGE_HANDLER(WM_CHAR, OnBlock)
    END_MSG_MAP()

    LRESULT OnKeyUp(UINT, WPARAM wparam, LPARAM lparam, BOOL& bHandled)
    {
        tstring key;
        m_table.recognize(wparam, lparam, &key);
        if (!key.empty())
        {
            SetWindowText(key.c_str());
            ::SendMessage(GetParent(), WM_USER, 0, 0); 
        }
        else
            bHandled = FALSE;
        return 0;
    }

    LRESULT OnBlock(UINT, WPARAM, LPARAM, BOOL& bHandled)
    {
        return 0;
    }
};

class PropertyHotkeys :  public CDialogImpl<PropertyHotkeys>
{ 
    PropertiesData *propData;
    PropertyListCtrl m_list;
    CBevelLine m_bl1;
    CBevelLine m_bl2;
    HotkeyBox m_hotkey;
    CEdit m_text;
    CButton m_add;
    CButton m_del;
    bool m_block;

public:
    enum { IDD = IDD_PROPERTY_HOTKEYS };
    PropertyHotkeys(PropertiesData *data) : propData(data), m_block(false) {}

private:
    BEGIN_MSG_MAP(PropertyHotkeys)
       MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
       MESSAGE_HANDLER(WM_USER, OnHotkeyEditChanged)
       COMMAND_ID_HANDLER(IDC_BUTTON_HOTKEY_ADD, OnAddElement)
       COMMAND_ID_HANDLER(IDC_BUTTON_HOTKEY_DEL, OnDeleteElement)       
       NOTIFY_HANDLER(IDC_LIST_HOTKEYS, LVN_ITEMCHANGED, OnListItemChanged)
       NOTIFY_HANDLER(IDC_LIST_HOTKEYS, NM_SETFOCUS, OnListItemChanged)
       NOTIFY_HANDLER(IDC_LIST_HOTKEYS, NM_KILLFOCUS, OnListKillFocus)
    END_MSG_MAP()


    LRESULT OnAddElement(WORD, WORD, HWND, BOOL&)
    {
        tstring hotkey, text;
        getWindowText(m_hotkey, &hotkey);
        getWindowText(m_text, &text);
        int index = propData->hotkeys.find(hotkey);
        if (index == -1)
        {
            int pos = m_list.GetItemCount();
            m_list.AddItem(pos, 0, hotkey.c_str());
            m_list.AddItem(pos, 1, text.c_str());
        }
        else
        {
            m_block = true;
            m_list.SetItem(index, 0, LVIF_TEXT, hotkey.c_str(), 0, 0, 0, NULL);
            m_list.SetItem(index, 1, LVIF_TEXT, text.c_str(), 0, 0, 0, NULL);            
        }
        propData->hotkeys.add(index, hotkey, text);
        if (index == -1)
            index = m_list.GetItemCount()-1;
        m_list.SelectItem(index);
        return 0;
    }

    LRESULT OnDeleteElement(WORD, WORD, HWND, BOOL&)
    {   
        int index = m_list.GetSelectedIndex();
        m_list.DeleteItem(index);        
        propData->hotkeys.del(index);
        return 0;
    }
    
    LRESULT OnHotkeyEditChanged(UINT, WPARAM, LPARAM, BOOL&)
    {
        int string_len = m_hotkey.GetWindowTextLength();
        m_add.EnableWindow(string_len == 0 ? FALSE : TRUE);
        return 0;
    }

    LRESULT OnListItemChanged(int , LPNMHDR , BOOL&)
    {
        int item_selected = m_list.GetSelectedIndex();
        if (item_selected == -1)
            m_del.EnableWindow(FALSE);
        else
            m_del.EnableWindow(TRUE);

        if (item_selected != -1 && !m_block)
        {
            const PropertiesData::hotkey_data& hotkey = propData->hotkeys.get(item_selected);
            m_hotkey.SetWindowText(hotkey.key.c_str());
            m_text.SetWindowText(hotkey.value.c_str());
            m_add.EnableWindow(TRUE);
        }
        m_block = false;
        return 0;
    }

    LRESULT OnListKillFocus(int , LPNMHDR , BOOL&)
    {
        if (GetFocus() != m_del)        
            m_del.EnableWindow(FALSE);            
        return 0;
    }

    LRESULT OnInitDialog(UINT, WPARAM, LPARAM, BOOL&)
	{
        m_hotkey.SubclassWindow(GetDlgItem(IDC_EDIT_HOTKEY));
        m_text.Attach(GetDlgItem(IDC_EDIT_HOTKEY_TEXT));
        m_add.Attach(GetDlgItem(IDC_BUTTON_HOTKEY_ADD));
        m_del.Attach(GetDlgItem(IDC_BUTTON_HOTKEY_DEL));
        m_list.Attach(GetDlgItem(IDC_LIST_HOTKEYS));
        m_list.addColumn(L"Hotkey", 20);
        m_list.addColumn(L"", 80);
        m_list.SetExtendedListViewStyle( m_list.GetExtendedListViewStyle() | LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT);
        m_bl1.SubclassWindow(GetDlgItem(IDC_STATIC_HOT_BL1));
        m_bl2.SubclassWindow(GetDlgItem(IDC_STATIC_HOT_BL2));
        m_add.EnableWindow(FALSE);
        m_del.EnableWindow(FALSE);

        for (int i=0,e=propData->hotkeys.size(); i<e; ++i)
        {
            const PropertiesData::action_data& hotkey = propData->hotkeys.get(i);
            m_list.AddItem(i, 0, hotkey.key.c_str());
            m_list.AddItem(i, 1, hotkey.value.c_str());
        }
        return 0;
    }
};
