#pragma once

class PropertySubs :  public CDialogImpl<PropertySubs>
{    
    PropertiesData *propData;
    PropertyListCtrl m_list;
    CBevelLine m_bl1;
    CBevelLine m_bl2;
    CEdit m_pattern;
    CEdit m_sub;
    CButton m_add;
    CButton m_del;
    CButton m_up;
    CButton m_down;
    bool m_block;

public:
    enum { IDD = IDD_PROPERTY_SUBS };
    PropertySubs(PropertiesData *data) : propData(data), m_block(false) {}

private:
    BEGIN_MSG_MAP(PropertySubs)
       MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
       COMMAND_ID_HANDLER(IDC_BUTTON_SUB_ADD, OnAddElement)
       COMMAND_ID_HANDLER(IDC_BUTTON_SUB_DEL, OnDeleteElement)
       COMMAND_ID_HANDLER(IDC_BUTTON_SUB_UP, OnUpElement)
       COMMAND_ID_HANDLER(IDC_BUTTON_SUB_DOWN, OnDownElement)
       COMMAND_HANDLER(IDC_EDIT_SUBS_PATTERN, EN_CHANGE, OnPatternEditChanged)
       NOTIFY_HANDLER(IDC_LIST_SUBS, LVN_ITEMCHANGED, OnListItemChanged)
       NOTIFY_HANDLER(IDC_LIST_SUBS, NM_SETFOCUS, OnListItemChanged)
       NOTIFY_HANDLER(IDC_LIST_SUBS, NM_KILLFOCUS, OnListKillFocus)
       REFLECT_NOTIFICATIONS()
    END_MSG_MAP()

    LRESULT OnAddElement(WORD, WORD, HWND, BOOL&)
    {
        tstring pattern, sub;
        getWindowText(m_pattern, &pattern);
        getWindowText(m_sub, &sub);
        int index = propData->subs.find(pattern);
        if (index == -1)
        {
            int pos = m_list.GetItemCount();
            m_list.AddItem(pos, 0, pattern.c_str());
            m_list.AddItem(pos, 1, sub.c_str());
        }
        else
        {
            m_block = true;
            m_list.SetItem(index, 0, LVIF_TEXT, pattern.c_str(), 0, 0, 0, NULL);
            m_list.SetItem(index, 1, LVIF_TEXT, sub.c_str(), 0, 0, 0, NULL);            
        }
        propData->subs.add(index, pattern, sub);
        if (index == -1)
            index = m_list.GetItemCount()-1;
        m_list.SelectItem(index);
        return 0;
    }

    LRESULT OnDeleteElement(WORD, WORD, HWND, BOOL&)
    {   
        int index = m_list.GetSelectedIndex();
        m_list.DeleteItem(index);        
        propData->subs.del(index);
        return 0;
    }

    void swapItems(int index1, int index2)
    {
        const PropertiesData::alias_data& i1 = propData->subs.get(index1);
        const PropertiesData::alias_data& i2 = propData->subs.get(index2);
        m_list.SetItemText(index1, 0, i2.key.c_str());
        m_list.SetItemText(index1, 1, i2.value.c_str());
        m_list.SetItemText(index2, 0, i1.key.c_str());
        m_list.SetItemText(index2, 1, i1.value.c_str());
        propData->subs.swap(index1, index2);
    }
    
    LRESULT OnUpElement(WORD, WORD, HWND, BOOL&)
    {
        int index = m_list.GetSelectedIndex();
        if (index > 0)
        {
            swapItems(index, index-1);
            m_list.SelectItem(index-1);
        }
        return 0;
    }

    LRESULT OnDownElement(WORD, WORD, HWND, BOOL&)
    {
        int index = m_list.GetSelectedIndex();
        int last = m_list.GetItemCount() - 1;
        if (index >= 0 && index != last)
        {
            swapItems(index, index+1);
            m_list.SelectItem(index+1);
        }
        return 0;
    }

    LRESULT OnPatternEditChanged(WORD, WORD, HWND, BOOL&)
    {
        int string_len = m_pattern.GetWindowTextLength();
        m_add.EnableWindow(string_len == 0 ? FALSE : TRUE);
        return 0;
    }

    LRESULT OnListItemChanged(int , LPNMHDR , BOOL&)
    {
        int item_selected = m_list.GetSelectedIndex();
        if (item_selected == -1)
        {
            m_del.EnableWindow(FALSE);
            m_up.EnableWindow(FALSE);
            m_down.EnableWindow(FALSE);
        }
        else
        {
            m_del.EnableWindow(TRUE);
            m_up.EnableWindow(TRUE);
            m_down.EnableWindow(TRUE);
        }

        if (item_selected != -1 && !m_block)
        {
            const PropertiesData::sub_data& sub = propData->subs.get(item_selected);
            m_pattern.SetWindowText(sub.key.c_str());
            m_sub.SetWindowText(sub.value.c_str());            
        }
        m_block = false;
        return 0;
    }    

    LRESULT OnListKillFocus(int , LPNMHDR , BOOL&)
    {
        if (GetFocus() != m_del)        
            m_del.EnableWindow(FALSE);          
        return 0;
    }

    LRESULT OnInitDialog(UINT, WPARAM, LPARAM, BOOL&)
	{
        m_pattern.Attach(GetDlgItem(IDC_EDIT_SUBS_PATTERN));
        m_sub.Attach(GetDlgItem(IDC_EDIT_SUB));
        m_add.Attach(GetDlgItem(IDC_BUTTON_SUB_ADD));
        m_del.Attach(GetDlgItem(IDC_BUTTON_SUB_DEL));
        m_up.Attach(GetDlgItem(IDC_BUTTON_SUB_UP));
        m_down.Attach(GetDlgItem(IDC_BUTTON_SUB_DOWN));
        m_list.Attach(GetDlgItem(IDC_LIST_SUBS));
        m_list.addColumn(L"", 50);
        m_list.addColumn(L"", 50);
        m_list.SetExtendedListViewStyle( m_list.GetExtendedListViewStyle() | LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT);
        m_bl1.SubclassWindow(GetDlgItem(IDC_STATIC_SUBSBL1));
        m_bl2.SubclassWindow(GetDlgItem(IDC_STATIC_SUBSBL2));
        m_add.EnableWindow(FALSE);
        m_del.EnableWindow(FALSE);
        m_up.EnableWindow(FALSE);
        m_down.EnableWindow(FALSE);

        for (int i=0,e=propData->subs.size(); i<e; ++i)
        {
            const PropertiesData::sub_data& sub = propData->subs.get(i);
            m_list.AddItem(i, 0, sub.key.c_str());
            m_list.AddItem(i, 1, sub.value.c_str());
        }
        return 0;
    }
};

