#pragma once

class PropertyListCtrlHandler
{
public:
    virtual void drawPropertyListItem(NMLVCUSTOMDRAW *pLVCD) = 0;
};

class PropertyListCtrl : public CWindowImpl<PropertyListCtrl, CListViewCtrl>
{
    float m_width_percent;
    PropertyListCtrlHandler *m_pHandler;
public:
    DECLARE_WND_SUPERCLASS(NULL, CListViewCtrl::GetWndClassName())
    PropertyListCtrl() : m_width_percent(0.0f), m_pHandler(NULL) {}

    void Attach(HWND hWnd) 
    {
        SubclassWindow(hWnd);    
        RECT pos;
        GetClientRect(&pos);
        float width = static_cast<float>(pos.right - GetSystemMetrics(SM_CXVSCROLL) - 1);
        m_width_percent = width / 100.0f;
    }

    void addColumn(const tstring& text, int width_percents)    
    {
        float width = m_width_percent * width_percents;
        int count = GetHeader().GetItemCount();                
        InsertColumn(count, text.c_str(), LVCFMT_LEFT, static_cast<int>(width));        
    }

    void setHandler(PropertyListCtrlHandler* handler)
    {
        m_pHandler = handler;
    }

private:
    BEGIN_MSG_MAP(PropertyListCtrl)    
       NOTIFY_CODE_HANDLER(NM_CUSTOMDRAW, OnCustomDraw) 
       REFLECTED_NOTIFY_CODE_HANDLER(NM_CUSTOMDRAW, OnCustomDraw)
    END_MSG_MAP()

    LRESULT OnCustomDraw(int, LPNMHDR pnmh, BOOL&)
    {
        if (!m_pHandler)
            return CDRF_DODEFAULT;

        LRESULT result = CDRF_DODEFAULT;
        NMLVCUSTOMDRAW* pLVCD = reinterpret_cast<NMLVCUSTOMDRAW*>( pnmh );
        DWORD stage = pLVCD->nmcd.dwDrawStage;

        // First thing - check the draw stage. If it's the control's prepaint
        // stage, then tell Windows we want messages for every item.
        if ( CDDS_PREPAINT == stage )
        {
            result = CDRF_NOTIFYITEMDRAW;
        }
        else if ( CDDS_ITEMPREPAINT == stage )
        {
            // This is the notification message for an item.  We'll request
            // notifications before each subitem's prepaint stage.
            result = CDRF_NOTIFYSUBITEMDRAW;
        }
        else if ( (CDDS_ITEMPREPAINT | CDDS_SUBITEM)  == stage )
        {
            // This is the prepaint stage for a subitem. Here's where we set the
            // item's text and background colors. Our return value will tell 
            // Windows to draw the subitem itself, but it will use the new colors
            // we set here.
            m_pHandler->drawPropertyListItem(pLVCD);
        }
        return result;
    }
};
